@@ -296,7 +296,7 @@ add_TTCIDHMetrics (pdf_obj *fontdict, struct tt_glyphs *g,
 
 static void
 add_TTCIDVMetrics (pdf_obj *fontdict, struct tt_glyphs *g,
-		   char *used_chars, unsigned char *cidtogidmap, unsigned short last_cid)
+		   char *used_chars, unsigned short last_cid)
 {
   pdf_obj *w2_array, *an_array = NULL;
   long cid;
@@ -542,9 +542,6 @@ CIDFont_type2_dofont (CIDFont *font)
     return;
   }
 
-#ifdef XETEX
-  sfont = sfnt_open(font->ft_face, SFNT_TYPE_TTC | SFNT_TYPE_TRUETYPE);
-#else
   fp = DPXFOPEN(font->ident, DPX_RES_TYPE_TTFONT);
   if (!fp) {
     fp = DPXFOPEN(font->ident, DPX_RES_TYPE_DFONT);
@@ -553,7 +550,6 @@ CIDFont_type2_dofont (CIDFont *font)
   } else {
     sfont = sfnt_open(fp);
   }
-#endif
 
   if (!sfont) {
     ERROR("Could not open TTF file: %s", font->ident);
@@ -566,13 +562,8 @@ CIDFont_type2_dofont (CIDFont *font)
       ERROR("Invalid TTC index in %s.", font->ident);
     break;
   case SFNT_TYPE_TRUETYPE:
-#ifdef XETEX
-	/* disable the check here becuase sfnt_open() does not distinguish dfont
-	 * from regular trutype */
-#else
     if (font->options->index > 0)
       ERROR("Found TrueType font file while expecting TTC file (%s).", font->ident);
-#endif
     offset = 0;
     break;
   case SFNT_TYPE_DFONT:
@@ -858,7 +849,7 @@ CIDFont_type2_dofont (CIDFont *font)
   } else {
     add_TTCIDHMetrics(font->fontdict, glyphs, used_chars, cidtogidmap, last_cid);
     if (v_used_chars)
-      add_TTCIDVMetrics(font->fontdict, glyphs, used_chars, cidtogidmap, last_cid);
+      add_TTCIDVMetrics(font->fontdict, glyphs, used_chars, last_cid);
   }
 
   tt_build_finish(glyphs);
@@ -947,11 +938,6 @@ CIDFont_type2_open (CIDFont *font, const char *name,
 
   ASSERT(font && opt);
 
-#ifdef XETEX
-  sfont = sfnt_open(font->ft_face, SFNT_TYPE_TTC | SFNT_TYPE_TRUETYPE);
-  if (!sfont)
-    return -1;
-#else
   fp = DPXFOPEN(name, DPX_RES_TYPE_TTFONT);
   if (!fp) {
     fp = DPXFOPEN(name, DPX_RES_TYPE_DFONT);
@@ -965,20 +951,14 @@ CIDFont_type2_open (CIDFont *font, const char *name,
     DPXFCLOSE(fp);
     return -1;
   }
-#endif
 
   switch (sfont->type) {
   case SFNT_TYPE_TTC:
     offset = ttc_read_offset(sfont, opt->index);
     break;
   case SFNT_TYPE_TRUETYPE:
-#ifdef XETEX
-	/* disable the check here becuase sfnt_open() does not distinguish dfont
-	 * from regular trutype */
-#else
     if (opt->index > 0)
       ERROR("Invalid TTC index (not TTC font): %s", name);
-#endif
     offset = 0;
     break;
   case SFNT_TYPE_DFONT:
@@ -989,7 +969,6 @@ CIDFont_type2_open (CIDFont *font, const char *name,
     if (fp)
       DPXFCLOSE(fp);
     return -1;
-    break;
   }
 
   if (sfnt_read_table_directory(sfont, offset) < 0) {
@@ -1113,9 +1092,3 @@ CIDFont_type2_open (CIDFont *font, const char *name,
 
   return 0;
 }
-
-void
-CIDFont_type2_release (CIDFont *font)
-{
-  return;
-}
