@@ -63,11 +63,11 @@
 
 #include "error.h"
 
+int is_xdv = 0;
+
 #ifdef XETEX
-int is_xetex = 1;
 const char *my_name = "xdvipdfmx";
 #else
-int is_xetex = 0;
 const char *my_name = "dvipdfmx";
 #endif
 
@@ -135,7 +135,7 @@ set_default_pdf_filename(void)
     pdf_filename[strlen(dvi_base)-4] = '\0';
   } else if (strlen(dvi_base) > 4 &&
              (FILESTRCASEEQ(".dvi", dvi_base+strlen(dvi_base)-4) ||
-              (is_xetex && FILESTRCASEEQ(".xdv", dvi_base+strlen(dvi_base)-4)))) {
+              FILESTRCASEEQ(".xdv", dvi_base+strlen(dvi_base)-4))) {
     pdf_filename = NEW(strlen(dvi_base)+1, char);
     strncpy(pdf_filename, dvi_base, strlen(dvi_base)-4);
     pdf_filename[strlen(dvi_base)-4] = '\0';
@@ -150,17 +150,15 @@ set_default_pdf_filename(void)
 static void
 show_version (void)
 {
-  printf ("This is %s Version " VERSION " by %s,\n",
-                   my_name,
-                   is_xetex ? "Jonathan Kew and Jin-Hwan Cho"
-                            : "the DVIPDFMx project team");
+  printf ("This is %s Version " VERSION " by the DVIPDFMx project team,\n",
+                   my_name);
   printf ("modified for TeX Live,\n");
-  if (is_xetex)
-    printf ("an extended version of DVIPDFMx, which in turn was\n");
+#ifdef XETEX
+  printf ("an extended version of DVIPDFMx, which in turn was\n");
+#endif
   printf ("an extended version of dvipdfm-0.13.2c developed by Mark A. Wicks.\n");
   printf ("\nCopyright (C) 2002-2014 the DVIPDFMx project team\n");
-  if (is_xetex)
-    printf ("Copyright (C) 2006-2014 SIL International and Jin-Hwan Cho.\n");
+  printf ("Copyright (C) 2006 SIL International.\n");
   printf ("\nThis is free software; you can redistribute it and/or modify\n");
   printf ("it under the terms of the GNU General Public License as published by\n");
   printf ("the Free Software Foundation; either version 2 of the License, or\n");
@@ -170,13 +168,12 @@ show_version (void)
 static void
 show_usage (void)
 {
-  printf ("\nUsage: %s [options] [dvifile[.dvi%s]]\n", my_name, is_xetex ? "|.xdv" : "");
+  printf ("\nUsage: %s [options] [dvifile[.dvi|.xdv]]\n", my_name);
   printf ("       %s --extractbb|--xbb|--ebb [options]\tBe \"extractbb\"\n", my_name);
   printf ("       %s --help|--version\n", my_name);
   printf ("\nOptions:\n"); 
   printf ("  -c \t\tIgnore color specials (for B&W printing)\n");
-  if (!is_xetex)
-    printf ("  --dvipdfm\tEnable DVIPDFM emulation mode\n");
+  printf ("  --dvipdfm\tEnable DVIPDFM emulation mode\n");
   printf ("  -d number\tSet PDF decimal digits (0-5) [2]\n");
   printf ("  -f filename\tSet font map file name [pdftex.map]\n");
   printf ("  -g dimension\tAnnotation \"grow\" amount [0.0in]\n");
@@ -217,7 +214,7 @@ show_usage (void)
   printf ("  -O number\tSet maximum depth of open bookmark items [0]\n");
   printf ("  -P number\tSet permission flags for PDF encryption [0x003C]\n");
   printf ("  -S \t\tEnable PDF encryption\n");
-  printf ("  -V number\tSet PDF minor version [4]\n");
+  printf ("  -V number\tSet PDF minor version [%d]\n", PDF_VERSION_DEFAULT);
   printf ("\nAll dimensions entered on the command line are \"true\" TeX dimensions.\n");
   printf ("Argument of \"-s\" lists physical page ranges separated by commas,\n");
   printf ("\te.g., \"-s 1-3,5-6\".\n");
@@ -347,17 +344,17 @@ select_pages (const char *pagespec)
     page_ranges[num_page_ranges].first = 0;
     page_ranges[num_page_ranges].last  = 0;
 
-    for ( ; *p && isspace(*p); p++);
+    for ( ; *p && isspace((unsigned char)*p); p++);
     q = parse_unsigned(&p, p + strlen(p)); /* Can't be signed. */
     if (q) { /* '-' is allowed here */
       page_ranges[num_page_ranges].first = atoi(q) - 1;
       page_ranges[num_page_ranges].last  = page_ranges[num_page_ranges].first;
       RELEASE(q);
     }
-    for ( ; *p && isspace(*p); p++);
+    for ( ; *p && isspace((unsigned char)*p); p++);
 
     if (*p == '-') {
-      for (++p; *p && isspace(*p); p++);
+      for (++p; *p && isspace((unsigned char)*p); p++);
       page_ranges[num_page_ranges].last = -1;
       if (*p) {
         q = parse_unsigned(&p, p + strlen(p));
@@ -365,7 +362,7 @@ select_pages (const char *pagespec)
           page_ranges[num_page_ranges].last = atoi(q) - 1;
           RELEASE(q);
         }
-        for ( ; *p && isspace(*p); p++);
+        for ( ; *p && isspace((unsigned char)*p); p++);
       }
     } else {
       page_ranges[num_page_ranges].last = page_ranges[num_page_ranges].first;
@@ -376,7 +373,7 @@ select_pages (const char *pagespec)
     if (*p == ',')
       p++;
     else  {
-      for ( ; *p && isspace(*p); p++);
+      for ( ; *p && isspace((unsigned char)*p); p++);
       if (*p)
         ERROR("Bad page range specification: %s", p);
     }
@@ -444,14 +441,16 @@ do_args (int argc, char *argv[])
           } else if (!strcmp(flag, "version")) {
             show_version();
             exit(0);
-          } else if (!is_xetex && !strcmp(flag, "dvipdfm")) {
+          } else if (!strcmp(flag, "dvipdfm")) {
             compat_mode = 1;
             goto Out_of_For_Loop;
           } else if (!strcmp(flag, "kpathsea-debug")) {
+            int value;
             CHECK_ARG(1, "kpathsearch debugging flags");
-            kpathsea_debug = atoi(argv[1]);
-            if (kpathsea_debug < 0)
+            value = atoi(argv[1]);
+            if (value < 0)
               ERROR("Invalid kpathsearch debugging flags specified: %s", argv[1]);
+            kpathsea_debug = value;
             POP_ARG();
             goto Out_of_For_Loop;
           }
@@ -539,7 +538,7 @@ do_args (int argc, char *argv[])
       {
         int ver_minor;
 
-        if (isdigit(*(flag+1))) {
+        if (isdigit((unsigned char)*(flag+1))) {
           flag++;
           ver_minor = atoi(flag);
         } else {
@@ -563,7 +562,7 @@ do_args (int argc, char *argv[])
       {
         int level;
 
-        if (isdigit(*(flag+1))) {
+        if (isdigit((unsigned char)*(flag+1))) {
           flag++;
           level = atoi(flag);
         } else {
@@ -575,7 +574,7 @@ do_args (int argc, char *argv[])
       }
       break;
       case 'd':
-        if (isdigit(*(flag+1))) {
+        if (isdigit((unsigned char)*(flag+1))) {
           flag++;
           pdfdecimaldigits = atoi(flag);
         } else {
@@ -748,7 +747,7 @@ error_cleanup (void)
 static void
 do_dvi_pages (void)
 {
-  long     page_no, page_count, i, step;
+  int      page_no, page_count, i, step;
   double   page_width, page_height;
   double   init_paper_width, init_paper_height;
   pdf_rect mediabox;
@@ -813,8 +812,7 @@ do_dvi_pages (void)
           mediabox.ury = page_height;
           pdf_doc_set_mediabox(page_count+1, &mediabox);
         }
-        dvi_do_page(page_no,
-                    page_width, page_height, x_offset, y_offset);
+        dvi_do_page(page_height, x_offset, y_offset);
         page_count++;
         MESG("]");
       }
@@ -916,7 +914,7 @@ main (int argc, char *argv[])
 
   if (argc > 1 &&
                (STREQ (argv[1], "--xbb") ||
-                (!is_xetex && STREQ (argv[1], "--dvipdfm")) ||
+                STREQ (argv[1], "--dvipdfm") ||
                 STREQ (argv[1], "--ebb"))) {
     argc--;
     base = argv++[1]+2;
@@ -943,7 +941,7 @@ main (int argc, char *argv[])
     exit(0);
   }
 
-  if (!is_xetex && FILESTRCASEEQ (base, "dvipdfm"))
+  if (FILESTRCASEEQ (base, "dvipdfm"))
     compat_mode = 1;
   else
     free (base);
