@@ -41,4 +41,9 @@ extern void WARN  (const char *fmt, ...);
 
 #define ASSERT(e) assert(e)
 
+#if defined(WIN32)
+#undef vfprintf
+#define vfprintf win32_vfprintf
+#endif
+
 #endif /* _ERROR_H_ */
