@@ -738,8 +738,7 @@ spc_handler_tpic_tx (struct spc_env *spe,
 
 
 static int
-spc_handler_tpic__init (struct spc_env *spe,
-                        struct spc_arg *ap, void *dp)
+spc_handler_tpic__init (struct spc_env *spe, void *dp)
 {
   struct spc_tpic_ *tp = dp;
 
@@ -763,8 +762,7 @@ spc_handler_tpic__init (struct spc_env *spe,
 }
 
 static int
-spc_handler_tpic__bophook (struct spc_env *spe,
-                           struct spc_arg *ap, void *dp)
+spc_handler_tpic__bophook (void *dp)
 {
   struct spc_tpic_ *tp = dp;
 
@@ -776,8 +774,7 @@ spc_handler_tpic__bophook (struct spc_env *spe,
 }
 
 static int
-spc_handler_tpic__eophook (struct spc_env *spe,
-                           struct spc_arg *ap, void *dp)
+spc_handler_tpic__eophook (struct spc_env *spe, void *dp)
 {
   struct spc_tpic_ *tp = dp;
 
@@ -791,8 +788,7 @@ spc_handler_tpic__eophook (struct spc_env *spe,
 }
 
 static int
-spc_handler_tpic__clean (struct spc_env *spe,
-                         struct spc_arg *ap, void *dp)
+spc_handler_tpic__clean (struct spc_env *spe, void *dp)
 {
   struct spc_tpic_ *tp = dp;
 
@@ -821,14 +817,14 @@ int
 spc_tpic_at_begin_page (void)
 {
   struct spc_tpic_ *tp = &_tpic_state;
-  return  spc_handler_tpic__bophook(NULL, NULL, tp);
+  return  spc_handler_tpic__bophook(tp);
 }
 
 int
 spc_tpic_at_end_page (void)
 {
   struct spc_tpic_ *tp = &_tpic_state;
-  return  spc_handler_tpic__eophook(NULL, NULL, tp);
+  return  spc_handler_tpic__eophook(NULL, tp);
 }
 
 
@@ -836,14 +832,14 @@ int
 spc_tpic_at_begin_document (void)
 {
   struct spc_tpic_ *tp = &_tpic_state;
-  return  spc_handler_tpic__init(NULL, NULL, tp);
+  return  spc_handler_tpic__init(NULL, tp);
 }
 
 int
 spc_tpic_at_end_document (void)
 {
   struct spc_tpic_ *tp = &_tpic_state;
-  return  spc_handler_tpic__clean(NULL, NULL, tp);
+  return  spc_handler_tpic__clean(NULL, tp);
 }
 
 
@@ -851,7 +847,7 @@ spc_tpic_at_end_document (void)
 #include "pdfparse.h" /* parse_val_ident :( */
 
 static pdf_obj *
-spc_parse_kvpairs (struct spc_env *spe, struct spc_arg *ap)
+spc_parse_kvpairs (struct spc_arg *ap)
 {
   pdf_obj *dict;
   char    *kp, *vp;
@@ -939,13 +935,13 @@ tpic_filter_getopts (pdf_obj *kp, pdf_obj *vp, void *dp)
 
 static int
 spc_handler_tpic__setopts (struct spc_env *spe,
-                           struct spc_arg *ap ) /* , void *dp) */
+                           struct spc_arg *ap)
 {
   struct spc_tpic_ *tp = &_tpic_state;
   pdf_obj  *dict;
   int       error = 0;
 
-  dict  = spc_parse_kvpairs(spe, ap);
+  dict  = spc_parse_kvpairs(ap);
   if (!dict)
     return  -1;
   error = pdf_foreach_dict(dict, tpic_filter_getopts, tp);
