@@ -422,7 +422,7 @@ release_charpath (t1_chardesc *cd)
  * Single byte operators:
  */
 static void
-do_operator1 (t1_chardesc *cd, card8 **data, card8 *endptr)
+do_operator1 (t1_chardesc *cd, card8 **data)
 {
   card8 op = **data;
 
@@ -635,7 +635,7 @@ do_othersubr0 (t1_chardesc *cd)
 
 /* Start flex */
 static void
-do_othersubr1 (t1_chardesc *cd)
+do_othersubr1 (void)
 {
   phase = T1_CS_PHASE_FLEX;
 }
@@ -679,7 +679,7 @@ do_othersubr3 (t1_chardesc *cd)
 }
 
 static void
-do_othersubr12 (t1_chardesc *cd)
+do_othersubr12 (void)
 {
   /* Othersubr12 call must immediately follow the hsbw or sbw. */
   if (phase != T1_CS_PHASE_INIT) {
@@ -786,10 +786,10 @@ do_callothersubr (t1_chardesc *cd)
 
   switch (subrno) {
   case 0:  do_othersubr0(cd) ; break;
-  case 1:  do_othersubr1(cd) ; break;
+  case 1:  do_othersubr1()   ; break;
   case 2:  do_othersubr2(cd) ; break;
   case 3:  do_othersubr3(cd) ; break;
-  case 12: do_othersubr12(cd); break;
+  case 12: do_othersubr12()  ; break;
   case 13: do_othersubr13(cd); break;
   default:
     ERROR("Unknown othersubr #%ld.", subrno);
@@ -1089,7 +1089,7 @@ t1char_build_charpath (t1_chardesc *cd,
     } else if (b0 == cs_escape) {
       do_operator2(cd, data, endptr);
     } else if (b0 < 32 && b0 != 28) { /* 19, 20 need mask */
-      do_operator1(cd, data, endptr);
+      do_operator1(cd, data);
     } else if ((b0 <= 22 && b0 >= 27) || b0 == 31) { /* reserved */
       status = CS_PARSE_ERROR; /* not an error ? */
     } else { /* integer */
@@ -1141,7 +1141,7 @@ do_postproc (t1_chardesc *cd)
   if ((b).lly > (y)) (b).lly = (y);\
   if ((b).ury < (y)) (b).ury = (y);\
 } while (0)
-#define TRY_COMPACT (prev && cur && ((prev->num_args + cur->num_args) <= CS_ARG_STACK_MAX))
+#define TRY_COMPACT (prev && cur && ((prev->num_args + cur->num_args) < CS_ARG_STACK_MAX))
 
   while (cur != NULL) {
     next = cur->next;
@@ -1405,7 +1405,7 @@ t1char_encode_charpath (t1_chardesc *cd,
       put_numbers(stem, 2, &dst, endptr);
       CHECK_STATUS();
       reset = 0;
-      if (2*num_hstems == CS_ARG_STACK_MAX) {
+      if (2*num_hstems > CS_ARG_STACK_MAX - 3) {
 	CHECK_BUFFER(1);
 	*dst++ = (card8) ((cd->flags & T1_CS_FLAG_USE_HINTMASK) ? cs_hstemhm : cs_hstem);
 	reset = 1;
@@ -1426,7 +1426,7 @@ t1char_encode_charpath (t1_chardesc *cd,
 	put_numbers(stem, 2, &dst, endptr);
 	CHECK_STATUS();
 	reset = 0;
-	if (2*num_vstems == CS_ARG_STACK_MAX) {
+	if (2*num_vstems > CS_ARG_STACK_MAX - 3) {
 	  CHECK_BUFFER(1);
 	  *dst++ = (card8) ((cd->flags & T1_CS_FLAG_USE_HINTMASK) ? cs_vstemhm : cs_vstem);
 	  reset = 1;
