@@ -215,13 +215,7 @@ tt_build_tables (sfnt *sfont, struct tt_glyphs *g)
 
   ASSERT(g);
 
-  if (sfont == NULL ||
-#ifdef XETEX
-      sfont->ft_face == NULL
-#else
-      sfont->stream == NULL
-#endif
-     )
+  if (sfont == NULL || sfont->stream == NULL)
     ERROR("File not opened.");
 
   if (sfont->type != SFNT_TYPE_TRUETYPE &&
@@ -529,13 +523,7 @@ tt_get_metrics (sfnt *sfont, struct tt_glyphs *g)
 
   ASSERT(g);
 
-  if (sfont == NULL ||
-#ifdef XETEX
-      sfont->ft_face == NULL
-#else
-      sfont->stream == NULL
-#endif
-     )
+  if (sfont == NULL || sfont->stream == NULL)
     ERROR("File not opened.");
 
   if (sfont->type != SFNT_TYPE_TRUETYPE &&
