@@ -2,7 +2,7 @@
 ** Bitmap.h                                                             **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef BITMAP_H
-#define BITMAP_H
+#ifndef DVISVGM_BITMAP_H
+#define DVISVGM_BITMAP_H
 
 #include <ostream>
 #include <vector>
@@ -35,8 +35,8 @@ class Bitmap
 			virtual void pixel (int x, int y, bool set, const Bitmap &bm) {}
 		};
 
-   public:
-      Bitmap ();
+	public:
+		Bitmap ();
 		Bitmap (int minx, int maxx, int miny , int maxy);
 		void resize (int minx, int maxx, int miny , int maxy);
 		void setBits(int r, int c, int n);
@@ -58,7 +58,7 @@ class Bitmap
 
 		std::ostream& write (std::ostream &os) const;
 
-   private:
+	private:
 		int _rows, _cols;     ///< number of rows, columns
 		int _xshift, _yshift; ///< horizontal/vertical shift
 		int _bpr;             ///< number of bytes per row
