@@ -2,7 +2,7 @@
 ** BoundingBox.cpp                                                      **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,6 +18,7 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#include <config.h>
 #include <algorithm>
 #include <sstream>
 #include <string>
@@ -60,7 +61,7 @@ BoundingBox::BoundingBox (const Length &ulxx, const Length &ulyy, const Length &
 
 
 BoundingBox::BoundingBox (const string &boxstr)
-	: _valid(false), _locked(false)
+	: _ulx(0), _uly(0), _lrx(0), _lry(0), _valid(false), _locked(false)
 {
 	set(boxstr);
 }
