@@ -2,7 +2,7 @@
 ** CmdLineParserBase.cpp                                                **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,6 +18,7 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#include <config.h>
 #include <algorithm>
 #include <cstdio>
 #include <cstring>
@@ -35,6 +36,8 @@ void CmdLineParserBase::init () {
 
 
 /** Parses all options given on the command line.
+ *  @param[in] argc number of command-line arguments
+ *  @param[in] argv array providing the command-line arguments
  *  @param[in] printErrors enable/disable printing of error messages */
 void CmdLineParserBase::parse (int argc, char **argv, bool printErrors) {
 	init();
@@ -71,13 +74,13 @@ void CmdLineParserBase::parse (int argc, char **argv, bool printErrors) {
 				do {
 					int shortname = ir.get();
 					if (const Option *opt = option(shortname)) {
-						if (!combined || opt->argmode == 0) {
-							if (opt->argmode == 'r' && strlen(argv[i]) == 2) { // required argument separated by whitespace?
+						if (!combined || opt->argmode == ARG_NONE) {
+							if (opt->argmode == ARG_REQUIRED && strlen(argv[i]) == 2) { // required argument separated by whitespace?
 								if (i+1 < argc && argv[i+1][0] != '-')
 									ib.assign(argv[++i]);
 							}
 							(*opt->handler)(this, ir, *opt, false);
-							if (opt->argmode == 0)
+							if (opt->argmode == ARG_NONE)
 								combined = true;
 						}
 						else {
@@ -131,7 +134,7 @@ void CmdLineParserBase::status () const {
 
 /** Returns the option information of a given short option name.
  *  If the option name can't be found 0 is returned.
- *  @param[in] longname long version of the option without leading hyphen (e.g. p, not -p) */
+ *  @param[in] shortname short version of the option without leading hyphen (e.g. p, not -p) */
 const CmdLineParserBase::Option* CmdLineParserBase::option (char shortname) const {
 	size_t numopts;  // number of available options
 	for (const Option *opts = options(&numopts); numopts > 0; ++opts) {
