@@ -2,7 +2,7 @@
 ** DVIActions.h                                                         **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef DVIACTIONS_H
-#define DVIACTIONS_H
+#ifndef DVISVGM_DVIACTIONS_H
+#define DVISVGM_DVIACTIONS_H
 
 #include <string>
 #include "Message.h"
@@ -32,24 +32,20 @@ class  SpecialManager;
 
 struct DVIActions
 {
-	static const double BP;
-	static const double IN;
-	static const double CM;
-	static const double MM;
 	virtual ~DVIActions () {}
-	virtual void setChar (double x, double y, unsigned c, const Font *f) {}
+	virtual void setChar (double x, double y, unsigned c, bool vertical, const Font *f) {}
 	virtual void setRule (double x, double y, double height, double width) {}
+	virtual void setTextOrientation (bool vertical) {}
 	virtual void moveToX (double x) {}
 	virtual void moveToY (double y) {}
 	virtual void defineFont (int num, const Font *font) {}
 	virtual void setFont (int num, const Font *font) {}
-	virtual void special (const std::string &s) {}
+	virtual void special (const std::string &s, double dvi2bp, bool preprocessing=false) {}
 	virtual void preamble (const std::string &cmt) {}
 	virtual void postamble () {}
-	virtual void beginPage (unsigned n, Int32 *c) {}
-	virtual void endPage () {}
+	virtual void beginPage (unsigned pageno, Int32 *c) {}
+	virtual void endPage (unsigned pageno) {}
 	virtual BoundingBox& bbox () =0;
-	virtual const SpecialManager* setProcessSpecials (const char *ignorelist) {return 0;}
 	virtual void progress (size_t current, size_t total, const char *id=0) {}
 };
 
