@@ -19,6 +19,7 @@
 *************************************************************************/
 
 #include <config.h>
+#include <algorithm>
 #include <cstdarg>
 #include <fstream>
 #include <iostream>
@@ -37,10 +38,10 @@
 
 using namespace std;
 
-bool DVIReader::COMPUTE_PAGE_LENGTH = false;
+bool DVIReader::COMPUTE_PROGRESS = false;
 
 
-DVIReader::DVIReader (istream &is, DVIActions *a) : StreamReader(is), _dviFormat(DVI_NONE), _actions(a)
+DVIReader::DVIReader (istream &is, DVIActions *a) : BasicDVIReader(is), _actions(a)
 {
 	_inPage = false;
 	_pageHeight = _pageWidth = 0;
@@ -48,13 +49,13 @@ DVIReader::DVIReader (istream &is, DVIActions *a) : StreamReader(is), _dviFormat
 	_tx = _ty = 0;    // no cursor translation
 	_prevYPos = numeric_limits<double>::min();
 	_inPostamble = false;
-	_totalPages = 0;  // we don't know the correct value yet
 	_currFontNum = 0;
 	_currPageNum = 0;
-	_pageLength = 0;
 	_pagePos = 0;
-	_prevBop = 0;
 	_mag = 1;
+	collectBopOffsets();
+	executePreamble();
+	executePostamble();
 }
 
 
@@ -65,136 +66,31 @@ DVIActions* DVIReader::replaceActions (DVIActions *a) {
 }
 
 
-/** Evaluates the next DVI command, and computes the corresponding handler.
- *  @param[in] compute_size if true, the size of variable-length parameters is computed
- *  @param[out] handler handler for current DVI command
- *  @param[out] length number of parameter bytes
- *  @param[out] param the handler must be called with this parameter
- *  @return opcode of current DVI command */
-int DVIReader::evalCommand (bool compute_size, CommandHandler &handler, int &length, int &param) {
-	struct DVICommand {
-		CommandHandler handler;
-		int length;  // number of parameter bytes
-	};
-
-   /* Each cmdFOO command reads the necessary number of bytes from the stream, so executeCommand
-   doesn't need to know the exact DVI command format. Some cmdFOO methods are used for multiple
-	DVI commands because they only differ in length of their parameters. */
-	static const DVICommand commands[] = {
-		{&DVIReader::cmdSetChar, 1}, {&DVIReader::cmdSetChar, 2}, {&DVIReader::cmdSetChar, 3}, {&DVIReader::cmdSetChar, 4}, // 128-131
-		{&DVIReader::cmdSetRule, 8},                                                                                        // 132
-		{&DVIReader::cmdPutChar, 1}, {&DVIReader::cmdPutChar, 2}, {&DVIReader::cmdPutChar, 3}, {&DVIReader::cmdPutChar, 4}, // 133-136
-		{&DVIReader::cmdPutRule, 8},                                                                                        // 137
-		{&DVIReader::cmdNop, 0},                                                                                            // 138
-		{&DVIReader::cmdBop, 44},    {&DVIReader::cmdEop, 0},                                                               // 139-140
-		{&DVIReader::cmdPush, 0},    {&DVIReader::cmdPop, 0},                                                               // 141-142
-		{&DVIReader::cmdRight, 1},   {&DVIReader::cmdRight, 2},   {&DVIReader::cmdRight, 3},   {&DVIReader::cmdRight, 4},   // 143-146
-		{&DVIReader::cmdW0, 0},                                                                                             // 147
-		{&DVIReader::cmdW, 1},       {&DVIReader::cmdW, 2},       {&DVIReader::cmdW, 3},       {&DVIReader::cmdW, 4},       // 148-151
-		{&DVIReader::cmdX0, 0},                                                                                             // 152
-		{&DVIReader::cmdX, 1},       {&DVIReader::cmdX, 2},       {&DVIReader::cmdX, 3},       {&DVIReader::cmdX, 4},       // 153-156
-		{&DVIReader::cmdDown, 1},    {&DVIReader::cmdDown, 2},    {&DVIReader::cmdDown, 3},    {&DVIReader::cmdDown, 4},    // 157-160
-		{&DVIReader::cmdY0, 0},                                                                                             // 161
-		{&DVIReader::cmdY, 1},       {&DVIReader::cmdY, 2},       {&DVIReader::cmdY, 3},       {&DVIReader::cmdY, 4},       // 162-165
-		{&DVIReader::cmdZ0, 0},                                                                                             // 166
-		{&DVIReader::cmdZ, 1},       {&DVIReader::cmdZ, 2},       {&DVIReader::cmdZ, 3},       {&DVIReader::cmdZ, 4},       // 167-170
-		{&DVIReader::cmdFontNum, 1}, {&DVIReader::cmdFontNum, 2}, {&DVIReader::cmdFontNum, 3}, {&DVIReader::cmdFontNum, 4}, // 235-238
-		{&DVIReader::cmdXXX, 1},     {&DVIReader::cmdXXX, 2},     {&DVIReader::cmdXXX, 3},     {&DVIReader::cmdXXX, 4},     // 239-242
-		{&DVIReader::cmdFontDef, 1}, {&DVIReader::cmdFontDef, 2}, {&DVIReader::cmdFontDef, 3}, {&DVIReader::cmdFontDef, 4}, // 243-246
-		{&DVIReader::cmdPre, 0},     {&DVIReader::cmdPost, 0},    {&DVIReader::cmdPostPost, 0},                             // 247-249
-		{&DVIReader::cmdXPic, 0},    {&DVIReader::cmdXFontDef, 0},{&DVIReader::cmdXGlyphA, 0}, {&DVIReader::cmdXGlyphS, 0}  // 251-254
-	};
-
-	const int opcode = in().get();
-	if (!in() || opcode < 0)  // at end of file
-		throw InvalidDVIFileException("invalid file");
-
-	param = -1;
-	if (opcode >= 0 && opcode <= 127) {
-		handler = &DVIReader::cmdSetChar0;
-		length = 0;
-		param = opcode;
-	}
-	else if (opcode >= 171 && opcode <= 234) {
-		handler = &DVIReader::cmdFontNum0;
-		length = 0;
-		param = opcode-171;
-	}
-	else if (opcode >= 251 && opcode <= 254 && _dviFormat == DVI_XDV) {  // XDV command?
-		static const CommandHandler handlers[] = {
-			&DVIReader::cmdXPic,
-			&DVIReader::cmdXFontDef,
-			&DVIReader::cmdXGlyphA,
-			&DVIReader::cmdXGlyphS
-		};
-		handler = handlers[opcode-251];
-		param = 0;
-	}
-	else if (opcode == 255 && _dviFormat == DVI_PTEX) {  // direction command set by pTeX?
-		handler = &DVIReader::cmdDir;
-		length = 1;
-	}
-	else if (opcode >= 250) {
-		ostringstream oss;
-		oss << "undefined DVI command (opcode " << opcode << ')';
-		throw DVIException(oss.str());
-	}
-	else {
-		const int offset = opcode <= 170 ? 128 : 235-(170-128+1);
-		handler = commands[opcode-offset].handler;
-		if (!compute_size)
-			length = commands[opcode-offset].length;
-		else {
-			if (opcode >= 239 && opcode <= 242) { // specials
-				int len = opcode-238;
-				UInt32 bytes = readUnsigned(len);
-				in().seekg(-len, ios_base::cur);
-				length = len+bytes;
-			}
-			else if (opcode >= 243 && opcode <= 246) { // fontdefs
-				int len = opcode-242;
-				len += 12;
-				in().seekg(len, ios_base::cur);   // skip fontnum, checksum, ssize, dsize
-				UInt32 bytes = readUnsigned(1);   // length of font path
-				bytes += readUnsigned(1);         // length of font name
-				in().seekg(-len-2, ios_base::cur);
-				length = len+bytes;
-			}
-			else
-				length = commands[opcode-offset].length;
-		}
-	}
-	if (param < 0)
-		param = length;
-	return opcode;
-}
-
-
 /** Reads a single DVI command from the current position of the input stream and calls the
  *  corresponding cmdFOO method.
- *  @return opcode of the executed command */
+ *  @return opcode of the command executed */
 int DVIReader::executeCommand () {
 	SignalHandler::instance().check();
 	CommandHandler handler;
-	int len;   // number of parameter bytes
 	int param; // parameter of handler
-	streampos pos = in().tellg();
-	int opcode = evalCommand(false, handler, len, param);
+	const streampos cmdpos = tell();
+	int opcode = evalCommand(handler, param);
 	(this->*handler)(param);
 	if (_dviState.v+_ty != _prevYPos) {
 		_tx = _ty = 0;
 		_prevYPos = _dviState.v;
 	}
-	if (COMPUTE_PAGE_LENGTH && _inPage && _actions) {
+	if (COMPUTE_PROGRESS && _inPage && _actions) {
+		size_t pagelen = numberOfPageBytes(_currPageNum-1);
 		// ensure progress() is called at 0%
-		if (opcode == 139) // bop?
-			_actions->progress(0, _pageLength);
+		if (opcode == 139)  // bop?
+			_actions->progress(0, pagelen);
 		// ensure progress() is called at 100%
-		if (in().peek() == 140)  // eop reached?
-			_pagePos = _pageLength;
+		if (peek() == 140)  // eop reached?
+			_pagePos = pagelen;
 		else
-			_pagePos += in().tellg()-pos;
-		_actions->progress(_pagePos, _pageLength);
+			_pagePos += tell()-cmdpos;
+		_actions->progress(_pagePos, pagelen);
 	}
 	return opcode;
 }
@@ -203,7 +99,7 @@ int DVIReader::executeCommand () {
 /** Executes all DVI commands read from the input stream. */
 void DVIReader::executeAll () {
 	int opcode = 0;
-	while (!in().eof() && opcode >= 0) {
+	while (!eof() && opcode >= 0) {
 		try {
 			opcode = executeCommand();
 		}
@@ -220,89 +116,73 @@ void DVIReader::executeAll () {
  *  @param[in] n number of page to be executed
  *  @returns true if page was read successfully */
 bool DVIReader::executePage (unsigned n) {
-	in().clear();    // reset all status bits
-	if (!in())
+	clearStream();    // reset all status bits
+	if (!isStreamValid())
 		throw DVIException("invalid DVI file");
-	in().seekg(-1, ios_base::end);      // stream pointer to last byte
-	while (in().peek() == 223)
-		in().seekg(-1, ios_base::cur);   // skip fill bytes
-	in().seekg(-4, ios_base::cur);      // now on first byte of q (pointer to begin of postamble)
-	UInt32 q = readUnsigned(4);         // pointer to begin of postamble
-	in().seekg(q, ios_base::beg);       // now on begin of postamble
-	if (executeCommand() != 248)        // execute postamble command but not the fontdefs
+	if (n < 1 || n > numberOfPages())
 		return false;
-	if (n < 1 || n > _totalPages)
-		return false;
-	in().seekg(_prevBop, ios_base::beg); // now on last bop
-	_inPostamble = false;                // we jumped out of the postamble
-	unsigned pageCount = _totalPages;
-	for (; pageCount > n && _prevBop > 0; pageCount--) {
-		in().seekg(41, ios_base::cur);    // skip bop and 10*4 \count bytes => now on pointer to prev bop
-		_prevBop = readSigned(4);
-		in().seekg(_prevBop, ios_base::beg);
-	}
+
+	seek(_bopOffsets[n-1]);           // goto bop of n-th page
+	_inPostamble = false;             // not in postamble
 	_currPageNum = n;
-	while (pageCount == n && executeCommand() != 140); // 140 == eop
-	return true;
-}
-
-
-bool DVIReader::executePages (unsigned first, unsigned last) {
-	in().clear();
-	if (!in())
-		throw DVIException("invalid DVI file");
-	if (first > last)
-		swap(first, last);
-	in().seekg(-1, ios_base::end);      // stream pointer to last byte
-	while (in().peek() == 223)
-		in().seekg(-1, ios_base::cur);   // skip fill bytes
-	in().seekg(-4, ios_base::cur);      // now on first byte of q (pointer to begin of postamble)
-	UInt32 q = readUnsigned(4);         // pointer to begin of postamble
-	in().seekg(q, ios_base::beg);       // now on begin of postamble
-	if (executeCommand() != 248)        // execute postamble command but not the fontdefs
-		return false;
-	first = max(1u, min(first, _totalPages));
-	last = max(1u, min(last, _totalPages));
-	in().seekg(_prevBop, ios_base::beg); // now on last bop
-	_inPostamble = false;                // we jumped out of the postamble
-	unsigned count = _totalPages;
-	for (; count > first && _prevBop > 0; count--) {
-		in().seekg(41, ios_base::cur);    // skip bop and 10*4 \count bytes => now on pointer to prev bop
-		_prevBop = readSigned(4);
-		in().seekg(_prevBop, ios_base::beg);
-	}
-	while (first <= last) {
-		_currPageNum = first++;
-		while (executeCommand () != 140); // 140 == eop
-	}
+	while (executeCommand() != 140);  // 140 == eop
 	return true;
 }
 
 
 void DVIReader::executePreamble () {
-	in().clear();
-	if (!in())
+	clearStream();
+	if (isStreamValid()) {
+		seek(0);
+		if (readByte() == 247) {
+			cmdPre(0);
+			return;
+		}
+	}
+	throw DVIException("invalid DVI file");
+}
+
+
+/** Moves stream pointer to begin of postamble */
+static void to_postamble (StreamReader &reader) {
+	reader.clearStream();
+	if (!reader.isStreamValid())
 		throw DVIException("invalid DVI file");
-	in().seekg(0, ios_base::beg);
-	if (in().get() != 247)
-		throw DVIException("invalid DVI file");
-	cmdPre(0);
+
+	reader.seek(-1, ios_base::end);     // stream pointer to last byte
+	int count=0;
+	while (reader.peek() == 223) {
+		reader.seek(-1, ios_base::cur);  // skip fill bytes
+		count++;
+	}
+	if (count < 4)  // the standard requires at least 4 trailing fill bytes
+		throw DVIException("missing fill bytes at end of file");
+
+	reader.seek(-4, ios_base::cur);     // now on first byte of q (pointer to begin of postamble)
+	UInt32 q = reader.readUnsigned(4);  // pointer to begin of postamble
+	reader.seek(q);                     // now on begin of postamble
 }
 
 
 /** Reads and executes the commands of the postamble. */
 void DVIReader::executePostamble () {
-	in().clear();  // reset all status bits
-	if (!in())
-		throw DVIException("invalid DVI file");
-	in().seekg(-1, ios_base::end);    // stream pointer to last byte
-	while (in().peek() == 223)
-		in().seekg(-1, ios_base::cur); // skip fill bytes
+	to_postamble(*this);
+	while (executeCommand() != 249);  // executes all commands until post_post (= 249) is reached
+}
 
-	in().seekg(-4, ios_base::cur);    // now on first byte of q (pointer to begin of postamble)
-	UInt32 q = readUnsigned(4);       // pointer to begin of postamble
-	in().seekg(q, ios_base::beg);     // now on begin of postamble
-	while (executeCommand() != 249);  // read all commands until postpost (= 249) is reached
+
+/** Collects and records the file offsets of all bop commands. */
+void DVIReader::collectBopOffsets () {
+	to_postamble(*this);
+	_bopOffsets.push_back(tell());     // also add offset of postamble
+	readByte();                        // skip post command
+	UInt32 offset = readUnsigned(4);   // offset of final bop
+	while ((Int32)offset > 0) {        // not yet on first bop?
+		_bopOffsets.push_back(offset);  // record offset
+		seek(offset+41);                // skip bop command and the 10 \count values => now on offset of previous bop
+		offset = readUnsigned(4);
+	}
+	reverse(_bopOffsets.begin(), _bopOffsets.end());
 }
 
 
@@ -320,35 +200,12 @@ double DVIReader::getYPos () const {
 }
 
 
-double DVIReader::getPageHeight () const {
-	return _pageHeight;
-}
-
-
-double DVIReader::getPageWidth () const {
-	return _pageWidth;
-}
-
-
-void DVIReader::verifyDVIFormat (int id) const {
-	switch (id) {
-		case DVI_STANDARD:
-		case DVI_PTEX:
-		case DVI_XDV:
-			break;
-		default:
-			ostringstream oss;
-			oss << "DVI format version " << id << " not supported";
-			throw DVIException(oss.str());
-	}
-}
-
 /////////////////////////////////////
 
-/** Reads and executes DVI preamble command. */
+/** Reads and executes DVI preamble command.
+ *  Format: pre ver[1] num[4] den[4] mag[4] cmtlen[1] cmt[cmtlen] */
 void DVIReader::cmdPre (int) {
-	_dviFormat = max(_dviFormat, (DVIFormat)readUnsigned(1)); // identification number
-	verifyDVIFormat(_dviFormat);
+	setDVIFormat((DVIFormat)readUnsigned(1)); // identification number
 	UInt32 num = readUnsigned(4);  // numerator units of measurement
 	UInt32 den = readUnsigned(4);  // denominator units of measurement
 	if (den == 0)
@@ -364,9 +221,10 @@ void DVIReader::cmdPre (int) {
 }
 
 
-/** Reads and executes DVI postamble command. */
+/** Reads and executes DVI postamble command.
+ *  Format: post p[4] num[4] den[4] mag[4] ph[4] pw[4] sd[2] np[2] */
 void DVIReader::cmdPost (int) {
-	_prevBop   = readUnsigned(4);  // pointer to previous bop
+	readUnsigned(4);  // skip pointer to previous bop
 	UInt32 num = readUnsigned(4);
 	UInt32 den = readUnsigned(4);
 	if (den == 0)
@@ -374,8 +232,10 @@ void DVIReader::cmdPost (int) {
 	_mag = readUnsigned(4);
 	_pageHeight = readUnsigned(4); // height of tallest page in dvi units
 	_pageWidth  = readUnsigned(4); // width of widest page in dvi units
-	readUnsigned(2);               // max. stack depth
-	_totalPages = readUnsigned(2); // total number of pages
+	readUnsigned(2);               // skip max. stack depth
+	if (readUnsigned(2) != numberOfPages())
+		throw DVIException("page count entry doesn't match actual number of pages");
+
 	// 1 dviunit * num/den == multiples of 0.0000001m
 	// 1 dviunit * _dvi2bp: length of 1 dviunit in PS points * _mag/1000
 	_dvi2bp = num/254000.0*72.0/den*_mag/1000.0;
@@ -387,42 +247,29 @@ void DVIReader::cmdPost (int) {
 }
 
 
-/** Reads and executes DVI postpost command. */
+/** Reads and executes DVI post_post command.
+ *  Format: post_post q[4] i[1] 223[>=4] */
 void DVIReader::cmdPostPost (int) {
 	_inPostamble = false;
 	readUnsigned(4);   // pointer to begin of postamble
-	_dviFormat = max(_dviFormat, (DVIFormat)readUnsigned(1));  // identification byte
-	verifyDVIFormat(_dviFormat);
+	setDVIFormat((DVIFormat)readUnsigned(1));  // identification byte
 	while (readUnsigned(1) == 223);  // skip fill bytes (223), eof bit should be set now
 }
 
 
-/** Reads and executes Begin-Of-Page command. */
+/** Reads and executes Begin-Of-Page command.
+ *  Format: bop c0[+4] ... c9[+4] p[+4] */
 void DVIReader::cmdBop (int) {
 	Int32 c[10];
 	for (int i=0; i < 10; i++)
 		c[i] = readSigned(4);
 	readSigned(4);        // pointer to peceeding bop (-1 in case of first page)
-	_dviState.reset();     // set all DVI registers to 0
+	_dviState.reset();    // set all DVI registers to 0
 	while (!_stateStack.empty())
 		_stateStack.pop();
 	_currFontNum = 0;
 	_inPage = true;
-	_pageLength = 0;
 	_pagePos = 0;
-	if (COMPUTE_PAGE_LENGTH) {
-		// compute number of bytes in current page
-		int length, param;
-		CommandHandler handler;
-		// read all commands until eop is found
-		while (evalCommand(true, handler, length, param) != 140) {
-			in().seekg(length, ios_base::cur);
-			_pageLength += length+1;  // parameter length + opcode length (1 byte)
-		}
-		++_pageLength;  // add length of eop command (1 byte)
-		in().seekg(-int(_pageLength), ios_base::cur);  // go back to first command following bop
-		_pageLength += 45; // add length of bop command
-	}
 	beginPage(_currPageNum, c);
 	if (_actions)
 		_actions->beginPage(_currPageNum, c);
@@ -473,10 +320,7 @@ void DVIReader::cmdPop (int) {
  * @throw DVIException if method is called ouside a bop/eop pair */
 void DVIReader::putChar (UInt32 c, bool moveCursor) {
 	if (!_inPage)
-		throw DVIException("set_char or put_char outside of page");
-
-	if (_actions && !_actions->fontProcessingEnabled())
-		return;
+		throw DVIException("set_char/put_char outside of page");
 
 	FontManager &fm = FontManager::instance();
 	Font *font = fm.getFont(_currFontNum);
@@ -504,7 +348,7 @@ void DVIReader::putChar (UInt32 c, bool moveCursor) {
 				//					Message::estream(true) << "invalid dvi in vf: " << e.getMessage() << endl; // @@
 			}
 			replaceStream(is);          // restore previous input stream
-			_dvi2bp = save_scale;  // restore previous scale factor
+			_dvi2bp = save_scale;       // restore previous scale factor
 			fm.leaveVF();               // restore previous font number context
 			cmdFontNum0(save_fontnum);  // restore previous font number
 			_dviState = pos;            // restore previous cursor position
@@ -538,8 +382,8 @@ void DVIReader::cmdSetChar0 (int c) {
  *  @param[in] len number of parameter bytes (possible values: 1-4)
  *  @throw DVIException if method is called ouside a bop/eop pair */
 void DVIReader::cmdSetChar (int len) {
-   // According to the dvi specification all character codes are unsigned
-   // except len == 4. At the moment all char codes are treated as unsigned...
+	// According to the dvi specification all character codes are unsigned
+	// except len == 4. At the moment all char codes are treated as unsigned...
 	UInt32 c = readUnsigned(len); // if len == 4 c may be signed
 	putChar(c, true);
 }
@@ -550,8 +394,8 @@ void DVIReader::cmdSetChar (int len) {
  *  @param[in] len number of parameter bytes (possible values: 1-4)
  *  @throw DVIException if method is called ouside a bop/eop pair */
 void DVIReader::cmdPutChar (int len) {
-   // According to the dvi specification all character codes are unsigned
-   // except len == 4. At the moment all char codes are treated as unsigned...
+	// According to the dvi specification all character codes are unsigned
+	// except len == 4. At the moment all char codes are treated as unsigned...
 	Int32 c = readUnsigned(len);
 	putChar(c, false);
 }
@@ -559,32 +403,30 @@ void DVIReader::cmdPutChar (int len) {
 
 /** Reads and executes set_rule command. Puts a solid rectangle at the current
  *  position and updates the cursor position.
+ *  Format: set_rule h[+4] w[+4]
  *  @throw DVIException if method is called ouside a bop/eop pair */
 void DVIReader::cmdSetRule (int) {
-	if (_inPage) {
-		double height = _dvi2bp*readSigned(4);
-		double width  = _dvi2bp*readSigned(4);
-		if (_actions && height > 0 && width > 0)
-			_actions->setRule(_dviState.h+_tx, _dviState.v+_ty, height, width);
-		moveRight(width);
-	}
-	else
+	if (!_inPage)
 		throw DVIException("set_rule outside of page");
+	double height = _dvi2bp*readSigned(4);
+	double width  = _dvi2bp*readSigned(4);
+	if (_actions && height > 0 && width > 0)
+		_actions->setRule(_dviState.h+_tx, _dviState.v+_ty, height, width);
+	moveRight(width);
 }
 
 
 /** Reads and executes set_rule command. Puts a solid rectangle at the current
  *  position but leaves the cursor position unchanged.
+ *  Format: put_rule h[+4] w[+4]
  *  @throw DVIException if method is called ouside a bop/eop pair */
 void DVIReader::cmdPutRule (int) {
-	if (_inPage) {
-		double height = _dvi2bp*readSigned(4);
-		double width  = _dvi2bp*readSigned(4);
-		if (_actions && height > 0 && width > 0)
-			_actions->setRule(_dviState.h+_tx, _dviState.v+_ty, height, width);
-	}
-	else
+	if (!_inPage)
 		throw DVIException("put_rule outside of page");
+	double height = _dvi2bp*readSigned(4);
+	double width  = _dvi2bp*readSigned(4);
+	if (_actions && height > 0 && width > 0)
+		_actions->setRule(_dviState.h+_tx, _dviState.v+_ty, height, width);
 }
 
 
@@ -649,14 +491,12 @@ void DVIReader::cmdDir (int) {
 
 
 void DVIReader::cmdXXX (int len) {
-	if (_inPage) {
-		UInt32 numBytes = readUnsigned(len);
-		string s = readString(numBytes);
-		if (_actions)
-			_actions->special(s, _dvi2bp);
-	}
-	else
+	if (!_inPage)
 		throw DVIException("special outside of page");
+	UInt32 numBytes = readUnsigned(len);
+	string s = readString(numBytes);
+	if (_actions)
+		_actions->special(s, _dvi2bp);
 }
 
 
@@ -664,9 +504,6 @@ void DVIReader::cmdXXX (int len) {
  * @param[in] num font number
  * @throw DVIException if font number is undefined */
 void DVIReader::cmdFontNum0 (int num) {
-	if (_actions && !_actions->fontProcessingEnabled())
-		return;
-
 	if (Font *font = FontManager::instance().getFont(num)) {
 		_currFontNum = num;
 		if (_actions && !dynamic_cast<VirtualFont*>(font))
@@ -696,8 +533,9 @@ void DVIReader::cmdFontNum (int len) {
  *  @param[in] ds design size in PS point units
  *  @param[in] ss scaled size in PS point units */
 void DVIReader::defineFont (UInt32 fontnum, const string &name, UInt32 cs, double ds, double ss) {
-	if (_actions && !_actions->fontProcessingEnabled())
+	if (!_inPostamble)  // only process font definitions collected in the postamble; skip all others
 		return;
+
 	FontManager &fm = FontManager::instance();
 	int id = fm.registerFont(fontnum, name, cs, ds, ss);
 	Font *font = fm.getFontById(id);
@@ -711,7 +549,7 @@ void DVIReader::defineFont (UInt32 fontnum, const string &name, UInt32 cs, doubl
 		fm.leaveVF();
 	}
 	if (_actions)
-	_actions->defineFont(id, font);
+		_actions->defineFont(id, font);
 }
 
 
@@ -724,10 +562,9 @@ void DVIReader::cmdFontDef (int len) {
 	UInt32 dsize    = readUnsigned(4);     // design size of font in DVI units
 	UInt32 pathlen  = readUnsigned(1);     // length of font path
 	UInt32 namelen  = readUnsigned(1);     // length of font name
-	string fontpath = readString(pathlen);
+	readString(pathlen);                   // skip font path
 	string fontname = readString(namelen);
 
-	// @@ TODO: avoid processing the font definitions in the postamble
 	defineFont(fontnum, fontname, checksum, dsize*_dvi2bp, ssize*_dvi2bp);
 }
 
@@ -753,7 +590,7 @@ void DVIReader::defineVFChar (UInt32 c, vector<UInt8> *dvi) {
 }
 
 
-/** XDV extension: include image or pdf file.
+/** XDV extension: includes image or pdf file.
  *  parameters: box[1] matrix[4][6] p[2] len[2] path[l] */
 void DVIReader::cmdXPic (int) {
 	// just skip the parameters
@@ -766,6 +603,7 @@ void DVIReader::cmdXPic (int) {
 }
 
 
+/** XDV extension: defines a native font */
 void DVIReader::cmdXFontDef (int) {
 	Int32 fontnum = readSigned(4);
 	double ptsize = _dvi2bp*readUnsigned(4);
@@ -782,8 +620,8 @@ void DVIReader::cmdXFontDef (int) {
 	}
 	if (flags & 0x0200) { // colored?
 		// The font color must not interfere with color specials. If the font color is not black,
-		// all color specials should be ignored, i.e. glyphs of a non-black font have a fixed color
-		// that can't be changed by colors specials.
+		// all color specials should be ignored, i.e. glyphs of a non-black fonts have a fixed color
+		// that can't be changed by color specials.
 		UInt32 rgba = readUnsigned(4);
 		color.set(UInt8(rgba >> 24), UInt8((rgba >> 16) & 0xff), UInt8((rgba >> 8) & 0xff));
 	}
@@ -798,21 +636,28 @@ void DVIReader::cmdXFontDef (int) {
 		for (int i=0; i < num_variations; i++)
 			readUnsigned(4);
 	}
-	if (_inPage && _actions && _actions->fontProcessingEnabled())
+	if (_inPage)
 		FontManager::instance().registerFont(fontnum, fontname, ptsize, style, color);
 }
 
 
+/** XDV extension: prints an array of characters where each character
+ *  can take independent x and y coordinates.
+ *  parameters: w[4] n[2] x[4][n] y[4][n] c[2][n] */
 void DVIReader::cmdXGlyphA (int) {
 	putGlyphArray(false);
 }
 
-
+/** XDV extension: prints an array/string of characters where each character
+ *  can take independent x coordinates whereas all share a single y coordinate.
+ *  parameters: w[4] n[2] x[4][n] y[4] c[2][n] */
 void DVIReader::cmdXGlyphS (int) {
 	putGlyphArray(true);
 }
 
 
+/** Implements the common functionality of cmdXGlyphA and cmdXGlyphS.
+ *  @param[in] xonly indicates if the characters share a single y coordinate (xonly==true) */
 void DVIReader::putGlyphArray (bool xonly) {
 	double strwidth = _dvi2bp*readSigned(4);
 	UInt16 num_glyphs = readUnsigned(2);
@@ -822,7 +667,9 @@ void DVIReader::putGlyphArray (bool xonly) {
 		x[i] = readSigned(4);
 		y[i] = xonly ? 0 : readSigned(4);
 	}
-	if (_actions && _actions->fontProcessingEnabled()) {
+	if (!_actions)
+		seek(2*num_glyphs, ios::cur);
+	else {
 		if (Font *font = FontManager::instance().getFont(_currFontNum)) {
 			for (int i=0; i < num_glyphs; i++) {
 				UInt16 glyph_index = readUnsigned(2);
@@ -832,9 +679,5 @@ void DVIReader::putGlyphArray (bool xonly) {
 			}
 		}
 	}
-	else {
-		for (int i=0; i < num_glyphs; i++)
-			readUnsigned(2);
-	}
 	moveRight(strwidth);
 }
