@@ -2,7 +2,7 @@
 ** DVIToSVG.h                                                           **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef DVITOSVG_H
-#define DVITOSVG_H
+#ifndef DVISVGM_DVITOSVG_H
+#define DVISVGM_DVITOSVG_H
 
 #include <iostream>
 #include <string>
@@ -28,36 +28,36 @@
 #include "SpecialManager.h"
 #include "SVGTree.h"
 
+class Matrix;
 struct SVGOutputBase;
 
 class DVIToSVG : public DVIReader
 {
-   public:
-      DVIToSVG (std::istream &is, SVGOutputBase &out);
+	public:
+		DVIToSVG (std::istream &is, SVGOutputBase &out);
 		~DVIToSVG ();
-		void convert (unsigned firstPage, unsigned lastPage, std::pair<int,int> *pageinfo=0);
-      void convert (const std::string &range, std::pair<int,int> *pageinfo=0);
-		const SpecialManager* setProcessSpecials (const char *ignorelist=0, bool pswarning=false);
-		const SpecialManager& specialManager () const    {return _specialManager;}
-		void setPageSize (const std::string &name)       {_bboxString = name;}
-		void setTransformation (const std::string &cmds) {_transCmds = cmds;}
+		void convert (const std::string &range, std::pair<int,int> *pageinfo=0);
+		void setPageSize (const std::string &name)           {_bboxString = name;}
+		void setPageTransformation (const std::string &cmds) {_transCmds = cmds;}
+		void getPageTransformation (Matrix &matrix) const;
+		std::string getSVGFilename (unsigned pageno) const;
+		static void setProcessSpecials (const char *ignorelist=0, bool pswarning=false);
 
-   public:
-      static char TRACE_MODE;
+	public:
+		static char TRACE_MODE;
 
 	protected:
 		DVIToSVG (const DVIToSVG &);
-		DVIToSVG operator = (const DVIToSVG &);
-		void beginPage (unsigned n, Int32 *c);
-		void endPage ();
+		void convert (unsigned firstPage, unsigned lastPage, std::pair<int,int> *pageinfo=0);
+		void beginPage (unsigned pageno, Int32 *c);
+		void endPage (unsigned pageno);
 		void embedFonts (XMLElementNode *svgElement);
 
-   private:
+	private:
 		SVGTree _svg;
 		SVGOutputBase &_out;
 		std::string _bboxString;
 		std::string _transCmds;
-		SpecialManager _specialManager;
 };
 
 #endif
