@@ -2,7 +2,7 @@
 ** Directory.h                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,28 +18,30 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef DIRECTORY_H
-#define DIRECTORY_H
+#ifndef DVISVGM_DIRECTORY_H
+#define DVISVGM_DIRECTORY_H
 
 #include <string>
 #ifdef __WIN32__
 	#include <windows.h>
-	#undef max
 #else
 	#include <dirent.h>
 #endif
 
 class Directory
 {
-   public:
+	public:
+		enum EntryType {ET_FILE, ET_DIR, ET_FILE_OR_DIR};
+
+	public:
 		Directory ();
-      Directory (std::string path);
+		Directory (std::string path);
 		~Directory ();
 		bool open (std::string path);
 		void close ();
-		const char* read (char type='a');
+		const char* read (EntryType type=ET_FILE_OR_DIR);
 
-   private:
+	private:
 		std::string _dirname;
 #ifdef __WIN32__
 		bool firstread;
