@@ -2,7 +2,7 @@
 ** FileFinder.cpp                                                       **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,21 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#undef __cplusplus
-#define __cplusplus 1
+#include <config.h>
+
+#ifdef MIKTEX
+	#include "MessageException.h"
+	#include "MiKTeXCom.h"
+	static MiKTeXCom *miktex=0;
+#else
+	#ifdef KPSE_CXX_UNSAFE
+	extern "C" {
+	#endif
+	#include <kpathsea/kpathsea.h>
+	#ifdef KPSE_CXX_UNSAFE
+	}
+	#endif
+#endif
 
 #include <cstdlib>
 #include <fstream>
@@ -29,23 +42,7 @@
 #include "FileSystem.h"
 #include "FontMap.h"
 #include "Message.h"
-
-using namespace std;
-
-#ifdef MIKTEX
-	#include "MessageException.h"
-	#include "MiKTeXCom.h"
-	static MiKTeXCom *miktex=0;
-#else
-	// unfortunately, the kpathsea headers are not C++-ready,
-	// so we have to wrap it with some ugly code
-	namespace KPS {
-		extern "C" {
-			#include <kpathsea/kpathsea.h>
-		}
-	}
-	using namespace KPS;
-#endif
+#include "Process.h"
 
 // ---------------------------------------------------
 
@@ -54,7 +51,7 @@ static bool _mktex_enabled = false;
 
 // ---------------------------------------------------
 
-static const char* find_file (const std::string &fname);
+static const char* find_file (const std::string &fname, const char *ftype);
 static const char* find_mapped_file (std::string fname);
 static const char* mktex (const std::string &fname);
 
@@ -76,7 +73,7 @@ void FileFinder::init (const char *argv0, const char *progname, bool enable_mkte
 	// enable tfm and mf generation (actually invoked by calls of kpse_make_tex)
 	kpse_set_program_enabled(kpse_tfm_format, 1, kpse_src_env);
 	kpse_set_program_enabled(kpse_mf_format, 1, kpse_src_env);
-	kpse_make_tex_discard_errors = false; // don't suppress messages of mktexFOO tools
+	kpse_make_tex_discard_errors = true;  // suppress messages from mktexFOO tools
 #ifdef TEXLIVEWIN32
 	texlive_gs_init();
 #endif
@@ -86,7 +83,7 @@ void FileFinder::init (const char *argv0, const char *progname, bool enable_mkte
 
 
 /** Cleans up the FileFinder. This function must be called before leaving the
- *  applications main() function. */
+ *  application's main() function. */
 void FileFinder::finish () {
 #ifdef MIKTEX
 	if (miktex) {
@@ -107,7 +104,7 @@ std::string FileFinder::version () {
 			init("", "", false);
 			autoinit = true;
 		}
-		string ret = miktex->getVersion();
+		std::string ret = miktex->getVersion();
 		if (autoinit)
 			finish();
 		return ret;
@@ -115,7 +112,6 @@ std::string FileFinder::version () {
 	catch (MessageException &e) {
 		if (autoinit)
 			finish();
-		return "not available";
 	}
 #else
 	if (const char *v = strrchr(KPSEVERSION, ' '))
@@ -128,20 +124,45 @@ std::string FileFinder::version () {
 /** Determines filetype by the filename extension and calls kpse_find_file
  *  to actually look up the file.
  *  @param[in] fname name of file to look up
+ *  @param[in] ftype expected file format of file fname; if 0, it's derived from the filename suffix
  *  @return file path on success, 0 otherwise */
-static const char* find_file (const std::string &fname) {
-	if (!_initialized)
+static const char* find_file (const std::string &fname, const char *ftype) {
+	if (!_initialized || fname.empty())
 		return 0;
 
-	size_t pos = fname.rfind('.');
-	if (pos == std::string::npos)
-		return 0;  // no extension => no search
-	const std::string ext  = fname.substr(pos+1);  // file extension
-#ifdef MIKTEX
-	return miktex->findFile(fname.c_str());
-#else
-	static std::string buf;
+	std::string ext;
+	if (ftype)
+		ext = ftype;
+	else {
+		size_t pos = fname.rfind('.');
+		if (pos == std::string::npos)
+			return 0;  // no extension and no file type => no search
+		ext = fname.substr(pos+1);
+	}
 
+	static std::string buf;
+#ifdef MIKTEX
+	if (ext == "dll" || ext == "exe") {
+		// lookup dll and exe files in the MiKTeX bin directory first
+		buf = miktex->getBinDir() + "/" + fname;
+		if (FileSystem::exists(buf.c_str()))
+			return buf.c_str();
+	}
+	else if (ext == "cmap") {
+		// The MiKTeX SDK doesn't support the lookup of files without suffix (yet), thus
+		// it's not possible to find cmap files which usually don't have a suffix. In order
+		// to work around this, we try to lookup the files by calling kpsewhich.
+		Process process("kpsewhich", std::string("-format=cmap ")+fname);
+		process.run(&buf);
+		return buf.empty() ? 0 : buf.c_str();
+	}
+	try {
+		return miktex->findFile(fname.c_str());
+	}
+	catch (const MessageException &e) {
+		return 0;
+	}
+#else
 #ifdef TEXLIVEWIN32
 	if (ext == "exe") {
 		// lookup exe files in directory where dvisvgm is located
@@ -152,20 +173,21 @@ static const char* find_file (const std::string &fname) {
 		return 0;
 	}
 #endif
-
 	static std::map<std::string, kpse_file_format_type> types;
 	if (types.empty()) {
-		types["tfm"] = kpse_tfm_format;
-		types["pfb"] = kpse_type1_format;
-		types["vf"]  = kpse_vf_format;
-		types["mf"]  = kpse_mf_format;
-		types["ttc"] = kpse_truetype_format;
-		types["ttf"] = kpse_truetype_format;
-		types["map"] = kpse_fontmap_format;
-		types["sty"] = kpse_tex_format;
-		types["enc"] = kpse_enc_format;
-		types["pro"] = kpse_tex_ps_header_format;
-      types["sfd"] = kpse_sfd_format;
+		types["tfm"]  = kpse_tfm_format;
+		types["pfb"]  = kpse_type1_format;
+		types["vf"]   = kpse_vf_format;
+		types["mf"]   = kpse_mf_format;
+		types["ttc"]  = kpse_truetype_format;
+		types["ttf"]  = kpse_truetype_format;
+		types["otf"]  = kpse_opentype_format;
+		types["map"]  = kpse_fontmap_format;
+		types["cmap"] = kpse_cmap_format;
+		types["sty"]  = kpse_tex_format;
+		types["enc"]  = kpse_enc_format;
+		types["pro"]  = kpse_tex_ps_header_format;
+		types["sfd"]  = kpse_sfd_format;
 	}
 	std::map<std::string, kpse_file_format_type>::iterator it = types.find(ext.c_str());
 	if (it == types.end())
@@ -187,7 +209,6 @@ static const char* find_file (const std::string &fname) {
 /** Checks whether the given file is mapped to a different name and if the
  *  file can be found under this name.
  *  @param[in] fname name of file to look up
- *  @param[in] fontmap font mappings
  *  @return file path on success, 0 otherwise */
 static const char* find_mapped_file (std::string fname) {
 	size_t pos = fname.rfind('.');
@@ -198,10 +219,10 @@ static const char* find_mapped_file (std::string fname) {
 	if (const FontMap::Entry *entry = FontMap::instance().lookup(base)) {
 		const char *path=0;
 		if (entry->fontname.find('.') != std::string::npos)  // does the mapped filename has an extension?
-			path = find_file(entry->fontname);           // look for that file
+			path = find_file(entry->fontname, 0);             // look for that file
 		else {                             // otherwise, use extension of unmapped file
 			fname = entry->fontname + "." + ext;
-			(path = find_file(fname)) || (path = mktex(fname));
+			(path = find_file(fname, 0)) || (path = mktex(fname));
 		}
 		return path;
 	}
@@ -227,9 +248,9 @@ static const char* mktex (const std::string &fname) {
 	const char *path = 0;
 #ifdef MIKTEX
 	// maketfm and makemf are located in miktex/bin which is in the search PATH
-	string toolname = (ext == "tfm" ? "miktex-maketfm" : "miktex-makemf");
+	std::string toolname = (ext == "tfm" ? "miktex-maketfm" : "miktex-makemf");
 	system((toolname+".exe "+fname).c_str());
-	path = find_file(fname);
+	path = find_file(fname, 0);
 #else
 	kpse_file_format_type type = (ext == "tfm" ? kpse_tfm_format : kpse_mf_format);
 	path = kpse_make_tex(type, fname.c_str());
@@ -246,11 +267,12 @@ static const char* mktex (const std::string &fname) {
  *  - in case of tfm or mf files: invokes the external mktextfm/mktexmf tool
  *    to create the missing file
  *  @param[in] fname name of file to look up
+ *  @param[in] ftype type/format of file to look up
  *  @param[in] extended if true, use fontmap lookup and mktexFOO calls
  *  @return path to file on success, 0 otherwise */
-const char* FileFinder::lookup (const std::string &fname, bool extended) {
+const char* FileFinder::lookup (const std::string &fname, const char *ftype, bool extended) {
 	const char *path;
-	if ((path = find_file(fname)) || (extended  && ((path = find_mapped_file(fname)) || (path = mktex(fname)))))
+	if ((path = find_file(fname, ftype)) || (extended  && ((path = find_mapped_file(fname)) || (path = mktex(fname)))))
 		return path;
 	return 0;
 }
