@@ -38,37 +38,16 @@
 #include "SignalHandler.h"
 #include "Subfont.h"
 #include "SVGTree.h"
+#include "Unicode.h"
 #include "macros.h"
 
 
 using namespace std;
 
 
-/** Returns true if 'unicode' is a valid unicode value in XML documents.
- *  XML version 1.0 doesn't allow various unicode character references
- *  (&#1; for example).  */
-static bool valid_unicode (UInt32 unicode) {
-	if ((unicode & 0xffff) == 0xfffe || (unicode & 0xffff) == 0xffff)
-		return false;
-
-	UInt32 ranges[] = {
-		0x0000, 0x0020,
-		0x007f, 0x0084,
-		0x0086, 0x009f,
-		0x202a, 0x202e,  // bidi control characters
-		0xd800, 0xdfff,
-		0xfdd0, 0xfdef,
-	};
-	for (size_t i=0; i < sizeof(ranges)/sizeof(UInt32)/2; i++)
-		if (unicode >= ranges[2*i] && unicode <= ranges[2*i+1])
-			return false;
-	return true;
-}
-
-
 UInt32 Font::unicode (UInt32 c) const {
 	// @@ this should be optimized :-)
-	return valid_unicode(c) ? c : 0x3400+c;
+	return Unicode::isValidCodepoint(c) ? c : 0x3400+c;
 }
 
 
@@ -119,6 +98,13 @@ void Font::getGlyphMetrics (int c, bool vertical, GlyphMetrics &metrics) const {
 }
 
 
+const char* Font::filename () const {
+	const char *fname = strrchr(path(), '/');
+	if (fname)
+		return fname+1;
+	return path();
+}
+
 ///////////////////////////////////////////////////////////////////////////////////////
 
 TFMFont::TFMFont (string name, UInt32 cs, double ds, double ss)
@@ -406,7 +392,7 @@ int PhysicalFont::traceAllGlyphs (bool includeCached, GFGlyphTracer::Callback *c
 				tracer.setGlyph(glyph);
 				for (int i=fchar; i <= lchar; i++) {
 					if (includeCached || !_cache.getGlyph(i)) {
-						glyph.newpath();
+						glyph.clear();
 						tracer.executeChar(i);
 						glyph.closeOpenSubPaths();
 						_cache.setGlyph(i, glyph);
@@ -524,16 +510,16 @@ UInt32 PhysicalFontImpl::unicode (UInt32 c) const {
 		return Font::unicode(c);
 
 	if (_localCharMap) {
-		if (UInt32 mapped_char = (*_localCharMap)[chr.number()])
+		if (UInt32 mapped_char = _localCharMap->valueAt(chr.number()))
 			return mapped_char;
 		// No unicode equivalent found in font file.
 		// Now we should look for a smart alternative but at the moment
 		// it's sufficient to simply choose a valid unused unicode value...
 		// Can we use the charcode itself as a unicode replacement?
-		if (!_localCharMap->valueExists(chr.number()) && valid_unicode(chr.number()))
+		if (!_localCharMap->valueExists(chr.number()) && Unicode::isValidCodepoint(chr.number()))
 			return chr.number();
 	}
-	if (valid_unicode(chr.number()))
+	if (Unicode::isValidCodepoint(chr.number()))
 		return chr.number();
 	return Font::unicode(chr.number());
 }
@@ -541,13 +527,13 @@ UInt32 PhysicalFontImpl::unicode (UInt32 c) const {
 
 /** Delete all temporary font files created by Metafont. */
 void PhysicalFontImpl::tidy () const {
-   if (type() == MF) {
+	if (type() == MF) {
 		const char *ext[] = {"gf", "tfm", "log", 0};
 		for (const char **p=ext; *p; ++p) {
 			if (FileSystem::exists((name()+"."+(*p)).c_str()))
 				FileSystem::remove(name()+"."+(*p));
 		}
-   }
+	}
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -612,23 +598,25 @@ double NativeFont::charDepth (int c) const {
 }
 
 
-UInt32 NativeFont::unicode (UInt32 c) const {
-	UInt32 ucode = decodeChar(c).number();
-	return valid_unicode(ucode) ? ucode : 0x3400+ucode;
-}
-
-
 bool NativeFontImpl::findAndAssignBaseFontMap () {
 	FontEngine &fe = FontEngine::instance();
 	fe.setFont(*this);
-	fe.buildCharMap(true, _toUnicodeMap);
-	_charmapID = fe.setUnicodeCharMap();
+	fe.setUnicodeCharMap();
+	fe.buildCharMap(_toUnicodeMap);
+	if (!_toUnicodeMap.addMissingMappings(fe.getNumGlyphs()))
+		Message::wstream(true) << "incomplete unicode mapping for native font " << name() << " (" << filename() << ")\n";
 	return true;
 }
 
 
 Character NativeFontImpl::decodeChar (UInt32 c) const {
-	return Character(Character::CHRCODE, _toUnicodeMap[c]);
+	return Character(Character::INDEX, c);
+}
+
+
+UInt32 NativeFontImpl::unicode (UInt32 c) const {
+	UInt32 ucode = _toUnicodeMap.valueAt(c);
+	return Unicode::isValidCodepoint(ucode) ? ucode : 0x3400+ucode;
 }
 
 //////////////////////////////////////////////////////////////////////////////
