@@ -28,10 +28,10 @@
 #include <string>
 #include <vector>
 #include "Character.h"
-#include "CharMap.h"
 #include "CharMapID.h"
 #include "Font.h"
 #include "Glyph.h"
+#include "RangeMap.h"
 #include "types.h"
 
 
@@ -39,7 +39,7 @@
  *  It's a wrapper for the Freetype font library. */
 class FontEngine
 {
-   public:
+	public:
 		~FontEngine ();
 		static FontEngine& instance ();
 		static std::string version ();
@@ -59,21 +59,22 @@ class FontEngine
 		int getFirstChar () const;
 		int getNextChar () const;
 		int getCharMapIDs (std::vector<CharMapID> &charmapIDs) const;
+		int getNumGlyphs () const;
 		CharMapID setUnicodeCharMap ();
 		CharMapID setCustomCharMap ();
 		std::vector<int> getPanose () const;
 		std::string getGlyphName (const Character &c) const;
 		int getCharByGlyphName (const char *name) const;
 		bool setCharMap (const CharMapID &charMapID);
-		void buildCharMap (bool invert, CharMap &charmap);
-		const CharMap* createCustomToUnicodeMap ();
+		void buildCharMap (RangeMap &charmap);
+		const RangeMap* createCustomToUnicodeMap ();
 
 	protected:
 		FontEngine ();
 		bool setFont (const std::string &fname, int fontindex, const CharMapID &charmapID);
 		int charIndex (const Character &c) const;
 
-   private:
+	private:
 		int _horDeviceRes, _vertDeviceRes;
 		mutable unsigned int _currentChar, _currentGlyphIndex;
 		FT_Face _currentFace;
