@@ -2,7 +2,7 @@
 ** FontManager.h                                                        **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef FONTMANAGER_H
-#define FONTMANAGER_H
+#ifndef DVISVGM_FONTMANAGER_H
+#define DVISVGM_FONTMANAGER_H
 
 #include <map>
 #include <ostream>
@@ -27,12 +27,13 @@
 #include <string>
 #include <stack>
 #include <vector>
+#include "Color.h"
+#include "FontStyle.h"
 #include "types.h"
 
 
 struct FileFinder;
 struct Font;
-class  TFM;
 class  VirtualFont;
 
 /** This class provides methods for easy DVI font handling.
@@ -48,10 +49,11 @@ class FontManager
 	typedef std::map<VirtualFont*, UInt32> VfFirstFontMap;
 	typedef std::stack<VirtualFont*> VfStack;
 
-   public:
-      ~FontManager ();
-      static FontManager& instance ();
+	public:
+		~FontManager ();
+		static FontManager& instance ();
 		int registerFont (UInt32 fontnum, std::string fontname, UInt32 checksum, double dsize, double scale);
+		int registerFont (UInt32 fontnum, std::string fname, double ptsize, const FontStyle &style, Color color);
 		Font* getFont (int n) const;
 		Font* getFont (const std::string &name) const;
 		Font* getFontById (int id) const;
@@ -67,10 +69,10 @@ class FontManager
 		const std::vector<Font*>& getFonts () const {return _fonts;}
 		std::ostream& write (std::ostream &os, Font *font=0, int level=0);
 
-   protected:
-      FontManager () {}
+	protected:
+		FontManager () {}
 
-   private:
+	private:
 		std::vector<Font*> _fonts; ///< all registered Fonts
 		Num2IdMap      _num2id;    ///< DVI font number -> fontID
 		Name2IdMap     _name2id;   ///< fontname -> fontID
