@@ -2,7 +2,7 @@
 ** GlyphTracerMessages.h                                                **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef GLYPHTRACERMESSAGES_H
-#define GLYPHTRACERMESSAGES_H
+#ifndef DVISVGM_GLYPHTRACERMESSAGES_H
+#define DVISVGM_GLYPHTRACERMESSAGES_H
 
 #include <sstream>
 #include "GFGlyphTracer.h"
