@@ -2,7 +2,7 @@
 ** GraphicPath.h                                                        **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,9 +18,10 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef GRAPHICPATH_H
-#define GRAPHICPATH_H
+#ifndef DVISVGM_GRAPHICPATH_H
+#define DVISVGM_GRAPHICPATH_H
 
+#include <cctype>
 #include <list>
 #include <ostream>
 #include <vector>
@@ -33,7 +34,9 @@
 template <typename T>
 class GraphicPath
 {
+	friend class PathClipper;
 	public:
+		enum WindingRule {WR_EVEN_ODD, WR_NON_ZERO};
 		typedef Pair<T> Point;
 
 		struct Command {
@@ -90,28 +93,38 @@ class GraphicPath
 			virtual void closepath () {}
 			virtual void draw (char cmd, const Point *points, int n) {}
 			virtual bool quit () {return false;}
+			virtual void finished () {}
 		};
 
 		typedef typename std::vector<Command>::iterator Iterator;
 		typedef typename std::vector<Command>::const_iterator ConstIterator;
 		typedef typename std::vector<Command>::const_reverse_iterator ConstRevIterator;
 
-   public:
-		void newpath () {
+	public:
+		GraphicPath (WindingRule wr=WR_NON_ZERO) : _windingRule(wr) {}
+
+		void setWindingRule (WindingRule wr) {_windingRule = wr;}
+		WindingRule windingRule () const     {return _windingRule;}
+
+		void clear () {
 			_commands.clear();
 		}
 
-		/// Returns true if path is empty (there is nothing to draw)
+		/// Returns true if the path is empty, i.e. there is nothing to draw
 		bool empty () const {
 			return _commands.empty();
 		}
 
-
 		/// Returns the number of path commands used to describe the path.
 		size_t size () const {
 			return _commands.size();
 		}
 
+		/// Insert another path at the beginning of this one.
+		void prepend (const GraphicPath &path) {
+			_commands.insert(_commands.begin(), path._commands.begin(), path._commands.end());
+		}
+
 		void moveto (const T &x, const T &y) {
 			moveto(Point(x, y));
 		}
@@ -182,29 +195,43 @@ class GraphicPath
 		 *  @param[in] os output stream used to write the SVG commands to
 		 *  @param[in] sx horizontal scale factor
 		 *  @param[in] sy vertical scale factor
-		 *  @param[in] dx horizontal translation in TeX point units
-		 *  @param[in] sy vertical translation in TeX point units */
-		void writeSVG (std::ostream &os, double sx=1.0, double sy=1.0, double dx=0.0, double dy=0.0) const {
+		 *  @param[in] dx horizontal translation in PS point units
+		 *  @param[in] dy vertical translation in PS point units */
+		void writeSVG (std::ostream &os, bool relative, double sx=1.0, double sy=1.0, double dx=0.0, double dy=0.0) const {
 			struct WriteActions : Actions {
-				WriteActions (std::ostream &os, double sx, double sy, double dx, double dy)
-					: _os(os), _sx(sx), _sy(sy), _dx(dx), _dy(dy) {}
+				WriteActions (std::ostream &os, bool relative, double sx, double sy, double dx, double dy)
+					: _os(os), _relative(relative), _sx(sx), _sy(sy), _dx(dx), _dy(dy) {}
 
 				void draw (char cmd, const Point *points, int n) {
+					if (_relative)
+						cmd = tolower(cmd);
 					_os << cmd;
 					switch (cmd) {
+						case 'h': _os << XMLString(_sx*(points->x()-_currentPoint.x())+_dx); break;
+						case 'v': _os << XMLString(_sy*(points->y()-_currentPoint.y())+_dy); break;
+						case 'z': _currentPoint = _startPoint; break;
 						case 'H': _os << XMLString(_sx*points->x()+_dx); break;
 						case 'V': _os << XMLString(_sy*points->y()+_dy); break;
 						default :
 							for (int i=0; i < n; i++) {
 								if (i > 0)
 									_os << ' ';
-								_os << XMLString(_sx*points[i].x()+_dx) << ' ' << XMLString(_sy*points[i].y()+_dy);
+								Point p = points[i];
+								if (_relative)
+									p -= _currentPoint;
+								_os << XMLString(_sx*p.x()+_dx) << ' ' << XMLString(_sy*p.y()+_dy);
 							}
 					}
+					if (cmd == 'm')
+						_startPoint = points[0];
+					if (islower(cmd) && n > 0)
+						_currentPoint = points[n-1];
 				}
 				std::ostream &_os;
+				bool _relative;
 				double _sx, _sy, _dx, _dy;
-			} actions(os, sx, sy, dx, dy);
+				Point _startPoint, _currentPoint;
+			} actions(os, relative, sx, sy, dx, dy);
 			iterate(actions, true);
 		}
 
@@ -253,8 +280,9 @@ class GraphicPath
 
 		void iterate (Actions &actions, bool optimize) const;
 
-   private:
+	private:
 		std::vector<Command> _commands;
+		WindingRule _windingRule;
 };
 
 
@@ -335,6 +363,7 @@ void GraphicPath<T>::iterate (Actions &actions, bool optimize) const {
 			cp = it->params[np-1];
 		prev = it;
 	}
+	actions.finished();
 }
 
 #endif
