@@ -2,7 +2,7 @@
 ** MessageException.h                                                   **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef MESSAGEEXCEPTION_H
-#define MESSAGEEXCEPTION_H
+#ifndef DVISVGM_MESSAGEEXCEPTION_H
+#define DVISVGM_MESSAGEEXCEPTION_H
 
 #include <exception>
 #include <string>
@@ -27,12 +27,12 @@
 
 class MessageException : public std::exception
 {
-   public:
-      MessageException (const std::string &msg) : _message(msg) {}
-      virtual ~MessageException () throw() {}
-      const char* what () const throw() {return _message.c_str();}
+	public:
+		MessageException (const std::string &msg) : _message(msg) {}
+		virtual ~MessageException () throw() {}
+		const char* what () const throw() {return _message.c_str();}
 
-   private:
+	private:
 		std::string _message;
 };
 
