@@ -72,64 +72,16 @@ bool PageRanges::parse (string str, int max_page) {
 			first = min(first, max_page);
 			last  = min(last, max_page);
 		}
-		insert(first, last);
+		addRange(first, last);
 	}
 	return true;
 }
 
 
-/** Adds a page range to the range collection.
- *  @param[in] first first page of new range
- *  @param[in] last last page of new range */
-void PageRanges::insert (int first, int last) {
-	if (first > last)
-		swap(first, last);
-	Container::iterator it = _ranges.begin();
-	while (it != _ranges.end() && first > it->first+1 && first > it->second+1)
-		++it;
-	if (it == _ranges.end() || last < it->first-1 || first > it->second+1)
-		it = _ranges.insert(it, Range(first, last));
-	else if ((first < it->first && last >= it->first-1) || (first <= it->second+1 && last > it->second)) {
-		it->first = min(it->first, first);
-		it->second = max(it->second, last);
-	}
-	// merge adjacent ranges
-	if (it != _ranges.end()) {
-		Container::iterator l = it;
-		Container::iterator r = it;
-		if (l == _ranges.begin())
-			l = _ranges.end();
-		else
-			--l;
-		++r;
-		bool l_modified = false;
-		bool r_modified = false;
-		if (l != _ranges.end() && l->second >= it->first-1) {
-			l->first = min(l->first, it->first);
-			l->second = max(l->second, it->second);
-			l_modified = true;
-		}
-		if (r != _ranges.end() && r->first <= it->second+1) {
-			r->first = min(r->first, it->first);
-			r->second = max(r->second, it->second);
-			r_modified = true;
-		}
-		if (l_modified || r_modified) {
-			_ranges.erase(it);
-			if (l_modified && r_modified && l->second >= r->first-1) {
-				l->first = min(l->first, r->first);
-				l->second = max(l->second, r->second);
-				_ranges.erase(r);
-			}
-		}
-	}
-}
-
-
 /** Returns the number of pages. */
-size_t PageRanges::pages () const {
+size_t PageRanges::numberOfPages () const {
 	size_t sum=0;
-	for (ConstIterator it=_ranges.begin(); it != _ranges.end(); ++it)
+	for (NumericRanges<int>::ConstIterator it=begin(); it != end(); ++it)
 		sum += it->second - it->first + 1;
 	return sum;
 }
