@@ -27,15 +27,18 @@
 #include "FileFinder.h"
 #include "Ghostscript.h"
 #include "Message.h"
+#include "PathClipper.h"
 #include "PSPattern.h"
 #include "PSPreviewFilter.h"
 #include "PsSpecialHandler.h"
 #include "SpecialActions.h"
+#include "SVGTree.h"
 #include "XMLNode.h"
 #include "XMLString.h"
 
 using namespace std;
 
+
 static inline double str2double (const string &str) {
 	double ret;
 	istringstream iss(str);
@@ -44,6 +47,9 @@ static inline double str2double (const string &str) {
 }
 
 
+bool PsSpecialHandler::COMPUTE_CLIPPATHS_INTERSECTIONS = false;
+
+
 PsSpecialHandler::PsSpecialHandler () : _psi(this), _actions(0), _previewFilter(_psi), _psSection(PS_NONE), _xmlnode(0)
 {
 }
@@ -74,6 +80,9 @@ void PsSpecialHandler::initialize () {
 		const char *headers[] = {"tex.pro", "texps.pro", "special.pro", /*"color.pro",*/ 0};
 		for (const char **p=headers; *p; ++p)
 			processHeaderFile(*p);
+		// disable bop/eop operators to prevent side-effects by
+		// unexpected bobs/eops present in PS specials
+		_psi.execute("\nTeXDict begin /bop{pop pop}def /eop{}def end ");
 		_psSection = PS_HEADERS;  // allow to process header specials now
 	}
 }
@@ -103,7 +112,7 @@ void PsSpecialHandler::enterBodySection () {
 		// push dictionary "TeXDict" with dvips definitions on dictionary stack
 		// and initialize basic dvips PostScript variables
 		oss << "\nTeXDict begin 0 0 1000 72 72 () @start 0 0 moveto ";
-  		if (_actions) {
+		if (_actions) {
 			float r, g, b;
 			_actions->getColor().getRGB(r, g, b);
 			oss << r << ' ' << g << ' ' << b << " setrgbcolor ";
@@ -126,10 +135,10 @@ void PsSpecialHandler::moveToDVIPos () {
 		const double x = _actions->getX();
 		const double y = _actions->getY();
 		ostringstream oss;
-      oss << '\n' << x << ' ' << y << " moveto ";
-      _psi.execute(oss.str());
-      _currentpoint = DPair(x, y);
-   }
+		oss << '\n' << x << ' ' << y << " moveto ";
+		_psi.execute(oss.str());
+		_currentpoint = DPair(x, y);
+	}
 }
 
 
@@ -149,11 +158,33 @@ static void exec_and_syncpos (PSInterpreter &psi, istream &is, const DPair &pos,
 }
 
 
+void PsSpecialHandler::preprocess (const char *prefix, istream &is, SpecialActions *actions) {
+	initialize();
+	if (_psSection != PS_HEADERS)
+		return;
+
+	_actions = actions;
+	if (*prefix == '!') {
+		_headerCode += "\n";
+		_headerCode += string(istreambuf_iterator<char>(is), istreambuf_iterator<char>());
+	}
+	else if (strcmp(prefix, "header=") == 0) {
+		// read and execute PS header file
+		string fname;
+		is >> fname;
+		processHeaderFile(fname.c_str());
+	}
+}
+
 
 bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions *actions) {
+	// process PS headers only once (in prescan)
+	if (*prefix == '!' || strcmp(prefix, "header=") == 0)
+		return true;
+
 	_actions = actions;
 	initialize();
-	if (_psSection != PS_BODY && *prefix != '!' && strcmp(prefix, "header=") != 0)
+	if (_psSection != PS_BODY)
 		enterBodySection();
 
 	if (*prefix == '"') {
@@ -163,20 +194,6 @@ bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions
 		_psi.execute(is);
 		_psi.execute("\n@endspecial ");
 	}
-	else if (*prefix == '!') {
-		if (_psSection == PS_HEADERS) {
-			_headerCode += "\n";
-			_headerCode += string(istreambuf_iterator<char>(is), istreambuf_iterator<char>());
-		}
-	}
-	else if (strcmp(prefix, "header=") == 0) {
-		if (_psSection == PS_HEADERS) {
-			// read and execute PS header file
-			string fname;
-			is >> fname;
-			processHeaderFile(fname.c_str());
-		}
-	}
 	else if (strcmp(prefix, "psfile=") == 0 || strcmp(prefix, "PSfile=") == 0) {
 		if (_actions) {
 			StreamInputReader in(is);
@@ -367,8 +384,13 @@ void PsSpecialHandler::dviEndPage (unsigned) {
 			_actions->bbox().lock();
 		}
 		Message::mstream() << "\napplying bounding box set by preview package (version " << _previewFilter.version() << ")\n";
-		if (horiz_baseline)
-			Message::mstream() << "width=" << XMLString(w) << "pt, " "height=" << XMLString(h) << "pt, " "depth=" << XMLString(d) << "pt\n";
+		if (horiz_baseline) {
+			const double bp2pt = 72.27/72.0;
+			Message::mstream() <<
+				"width=" << XMLString(w*bp2pt) << "pt, "
+				"height=" << XMLString(h*bp2pt) << "pt, "
+				"depth=" << XMLString(d*bp2pt) << "pt\n";
+		}
 		else
 			Message::mstream() << "can't determine height, width, and depth due to non-horizontal baseline\n";
 	}
@@ -429,78 +451,80 @@ void PsSpecialHandler::closepath (vector<double> &p) {
 /** Draws the current path recorded by previously executed path commands (moveto, lineto,...).
  *  @param[in] p not used */
 void PsSpecialHandler::stroke (vector<double> &p) {
-	if (!_path.empty() && _actions) {
-		BoundingBox bbox;
-		if (!_actions->getMatrix().isIdentity()) {
-			_path.transform(_actions->getMatrix());
-			if (!_xmlnode)
-				bbox.transform(_actions->getMatrix());
-		}
+	if ((_path.empty() && !_clipStack.clippathLoaded()) || !_actions)
+		return;
 
-		XMLElementNode *path=0;
-		Pair<double> point;
-		if (_path.isDot(point)) {  // zero-length path?
-			if (_linecap == 1) {    // round line ends?  => draw dot
-				double x = point.x();
-				double y = point.y();
-				double r = _linewidth/2.0;
-				path = new XMLElementNode("circle");
-				path->addAttribute("cx", x);
-				path->addAttribute("cy", y);
-				path->addAttribute("r", r);
-				path->addAttribute("fill", _actions->getColor().rgbString());
-				bbox = BoundingBox(x-r, y-r, x+r, y+r);
-			}
-		}
-		else {
-			// compute bounding box
-			_path.computeBBox(bbox);
-			bbox.expand(_linewidth/2);
-
-			ostringstream oss;
-			_path.writeSVG(oss);
-			path = new XMLElementNode("path");
-			path->addAttribute("d", oss.str());
-			path->addAttribute("stroke", _actions->getColor().rgbString());
-			path->addAttribute("fill", "none");
-			if (_linewidth != 1)
-				path->addAttribute("stroke-width", _linewidth);
-			if (_miterlimit != 4)
-				path->addAttribute("stroke-miterlimit", _miterlimit);
-			if (_linecap > 0)     // default value is "butt", no need to set it explicitly
-				path->addAttribute("stroke-linecap", _linecap == 1 ? "round" : "square");
-			if (_linejoin > 0)    // default value is "miter", no need to set it explicitly
-				path->addAttribute("stroke-linejoin", _linecap == 1 ? "round" : "bevel");
-			if (_opacityalpha < 1)
-				path->addAttribute("stroke-opacity", _opacityalpha);
-			if (!_dashpattern.empty()) {
-				ostringstream oss;
-				for (size_t i=0; i < _dashpattern.size(); i++) {
-					if (i > 0)
-						oss << ',';
-					oss << XMLString(_dashpattern[i]);
-				}
-				path->addAttribute("stroke-dasharray", oss.str());
-				if (_dashoffset != 0)
-					path->addAttribute("stroke-dashoffset", _dashoffset);
-			}
-		}
-		if (path && _clipStack.top()) {
-			// assign clipping path and clip bounding box
-			path->addAttribute("clip-path", XMLString("url(#clip")+XMLString(_clipStack.topID())+")");
-			BoundingBox clipbox;
-			_clipStack.top()->computeBBox(clipbox);
-			bbox.intersect(clipbox);
-		}
-
-		if (_xmlnode)
-			_xmlnode->append(path);
-		else {
-			_actions->appendToPage(path);
-			_actions->embed(bbox);
-		}
-		_path.newpath();
+	BoundingBox bbox;
+	if (!_actions->getMatrix().isIdentity()) {
+		_path.transform(_actions->getMatrix());
+		if (!_xmlnode)
+			bbox.transform(_actions->getMatrix());
 	}
+	if (_clipStack.clippathLoaded() && _clipStack.top())
+		_path.prepend(*_clipStack.top());
+	XMLElementNode *path=0;
+	Pair<double> point;
+	if (_path.isDot(point)) {  // zero-length path?
+		if (_linecap == 1) {    // round line ends?  => draw dot
+			double x = point.x();
+			double y = point.y();
+			double r = _linewidth/2.0;
+			path = new XMLElementNode("circle");
+			path->addAttribute("cx", x);
+			path->addAttribute("cy", y);
+			path->addAttribute("r", r);
+			path->addAttribute("fill", _actions->getColor().rgbString());
+			bbox = BoundingBox(x-r, y-r, x+r, y+r);
+		}
+	}
+	else {
+		// compute bounding box
+		_path.computeBBox(bbox);
+		bbox.expand(_linewidth/2);
+
+		ostringstream oss;
+		_path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
+		path = new XMLElementNode("path");
+		path->addAttribute("d", oss.str());
+		path->addAttribute("stroke", _actions->getColor().rgbString());
+		path->addAttribute("fill", "none");
+		if (_linewidth != 1)
+			path->addAttribute("stroke-width", _linewidth);
+		if (_miterlimit != 4)
+			path->addAttribute("stroke-miterlimit", _miterlimit);
+		if (_linecap > 0)     // default value is "butt", no need to set it explicitly
+			path->addAttribute("stroke-linecap", _linecap == 1 ? "round" : "square");
+		if (_linejoin > 0)    // default value is "miter", no need to set it explicitly
+			path->addAttribute("stroke-linejoin", _linecap == 1 ? "round" : "bevel");
+		if (_opacityalpha < 1)
+			path->addAttribute("stroke-opacity", _opacityalpha);
+		if (!_dashpattern.empty()) {
+			ostringstream oss;
+			for (size_t i=0; i < _dashpattern.size(); i++) {
+				if (i > 0)
+					oss << ',';
+				oss << XMLString(_dashpattern[i]);
+			}
+			path->addAttribute("stroke-dasharray", oss.str());
+			if (_dashoffset != 0)
+				path->addAttribute("stroke-dashoffset", _dashoffset);
+		}
+	}
+	if (path && _clipStack.top()) {
+		// assign clipping path and clip bounding box
+		path->addAttribute("clip-path", XMLString("url(#clip")+XMLString(_clipStack.topID())+")");
+		BoundingBox clipbox;
+		_clipStack.top()->computeBBox(clipbox);
+		bbox.intersect(clipbox);
+		_clipStack.setClippathLoaded(false);
+	}
+	if (_xmlnode)
+		_xmlnode->append(path);
+	else {
+		_actions->appendToPage(path);
+		_actions->embed(bbox);
+	}
+	_path.clear();
 }
 
 
@@ -508,43 +532,47 @@ void PsSpecialHandler::stroke (vector<double> &p) {
  *  @param[in] p not used
  *  @param[in] evenodd true: use even-odd fill algorithm, false: use nonzero fill algorithm */
 void PsSpecialHandler::fill (vector<double> &p, bool evenodd) {
-	if (!_path.empty() && _actions) {
-		// compute bounding box
-		BoundingBox bbox;
-		_path.computeBBox(bbox);
-		if (!_actions->getMatrix().isIdentity()) {
-			_path.transform(_actions->getMatrix());
-			if (!_xmlnode)
-				bbox.transform(_actions->getMatrix());
-		}
+	if ((_path.empty() && !_clipStack.clippathLoaded()) || !_actions)
+		return;
 
-		ostringstream oss;
-		_path.writeSVG(oss);
-		XMLElementNode *path = new XMLElementNode("path");
-		path->addAttribute("d", oss.str());
-		if (_pattern)
-			path->addAttribute("fill", XMLString("url(#")+_pattern->svgID()+")");
-		else if (_actions->getColor() != Color::BLACK || _savenode)
-			path->addAttribute("fill", _actions->getColor().rgbString());
-		if (_clipStack.top()) {
-			// assign clipping path and clip bounding box
-			path->addAttribute("clip-path", XMLString("url(#clip")+XMLString(_clipStack.topID())+")");
-			BoundingBox clipbox;
-			_clipStack.top()->computeBBox(clipbox);
-			bbox.intersect(clipbox);
-		}
-		if (evenodd)  // SVG default fill rule is "nonzero" algorithm
-			path->addAttribute("fill-rule", "evenodd");
-		if (_opacityalpha < 1)
-			path->addAttribute("fill-opacity", _opacityalpha);
-		if (_xmlnode)
-			_xmlnode->append(path);
-		else {
-			_actions->appendToPage(path);
-			_actions->embed(bbox);
-		}
-		_path.newpath();
+	// compute bounding box
+	BoundingBox bbox;
+	_path.computeBBox(bbox);
+	if (!_actions->getMatrix().isIdentity()) {
+		_path.transform(_actions->getMatrix());
+		if (!_xmlnode)
+			bbox.transform(_actions->getMatrix());
 	}
+	if (_clipStack.clippathLoaded() && _clipStack.top())
+		_path.prepend(*_clipStack.top());
+
+	ostringstream oss;
+	_path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
+	XMLElementNode *path = new XMLElementNode("path");
+	path->addAttribute("d", oss.str());
+	if (_pattern)
+		path->addAttribute("fill", XMLString("url(#")+_pattern->svgID()+")");
+	else if (_actions->getColor() != Color::BLACK || _savenode)
+		path->addAttribute("fill", _actions->getColor().rgbString());
+	if (_clipStack.top()) {
+		// assign clipping path and clip bounding box
+		path->addAttribute("clip-path", XMLString("url(#clip")+XMLString(_clipStack.topID())+")");
+		BoundingBox clipbox;
+		_clipStack.top()->computeBBox(clipbox);
+		bbox.intersect(clipbox);
+		_clipStack.setClippathLoaded(false);
+	}
+	if (evenodd)  // SVG default fill rule is "nonzero" algorithm
+		path->addAttribute("fill-rule", "evenodd");
+	if (_opacityalpha < 1)
+		path->addAttribute("fill-opacity", _opacityalpha);
+	if (_xmlnode)
+		_xmlnode->append(path);
+	else {
+		_actions->appendToPage(path);
+		_actions->embed(bbox);
+	}
+	_path.clear();
 }
 
 
@@ -645,45 +673,77 @@ void PsSpecialHandler::setpattern (vector<double> &p) {
 
 /** Clears the current clipping path.
  *  @param[in] p not used */
-void PsSpecialHandler::initclip (vector<double> &p) {
-	_clipStack.push();  // push empty path
+void PsSpecialHandler::initclip (vector<double> &) {
+	_clipStack.pushEmptyPath();
 }
 
 
-/** Assigns a new clipping path.
+/** Assigns the current clipping path to the graphics path. */
+void PsSpecialHandler::clippath (std::vector<double>&) {
+	if (!_clipStack.empty()) {
+		_clipStack.setClippathLoaded(true);
+		_path.clear();
+	}
+}
+
+
+/** Assigns a new clipping path to the graphics state using the current path.
+ *  If the graphics state already contains a clipping path, the new one is
+ *  computed by intersecting the current clipping path with the current graphics
+ *  path (see PS language reference, 3rd edition, pp. 193, 542)
  *  @param[in] p not used
  *  @param[in] evenodd true: use even-odd fill algorithm, false: use nonzero fill algorithm */
-void PsSpecialHandler::clip (vector<double> &p, bool evenodd) {
+void PsSpecialHandler::clip (vector<double> &, bool evenodd) {
 	// when this method is called, _path contains the clipping path
-	if (!_path.empty() && _actions) {
-		if (!_actions->getMatrix().isIdentity())
-			_path.transform(_actions->getMatrix());
+	if (_path.empty() || !_actions)
+		return;
 
-		int oldID = _clipStack.topID();
+	Path::WindingRule windingRule = evenodd ? Path::WR_EVEN_ODD : Path::WR_NON_ZERO;
+	_path.setWindingRule(windingRule);
+
+	if (!_actions->getMatrix().isIdentity())
+		_path.transform(_actions->getMatrix());
+
+	int oldID = _clipStack.topID();
+
+	ostringstream oss;
+	if (!COMPUTE_CLIPPATHS_INTERSECTIONS || oldID < 1) {
 		_clipStack.replace(_path);
-		int newID = _clipStack.topID();
-
-		ostringstream oss;
-		_path.writeSVG(oss);
-		XMLElementNode *path = new XMLElementNode("path");
-		path->addAttribute("d", oss.str());
-		if (evenodd)
-			path->addAttribute("clip-rule", "evenodd");
-
-		XMLElementNode *clip = new XMLElementNode("clipPath");
-		clip->addAttribute("id", XMLString("clip")+XMLString(newID));
-		if (oldID)
-			clip->addAttribute("clip-path", XMLString("url(#clip")+XMLString(oldID)+")");
-
-		clip->append(path);
-		_actions->appendToDefs(clip);
+		_path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
 	}
+	else {
+		// compute the intersection of the current clipping path with the current graphics path
+		Path *oldPath = _clipStack.getPath(oldID);
+		Path intersectedPath(windingRule);
+		PathClipper clipper;
+		clipper.intersect(*oldPath, _path, intersectedPath);
+		_clipStack.replace(intersectedPath);
+		intersectedPath.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
+	}
+
+	XMLElementNode *pathElem = new XMLElementNode("path");
+	pathElem->addAttribute("d", oss.str());
+	if (evenodd)
+		pathElem->addAttribute("clip-rule", "evenodd");
+
+	int newID = _clipStack.topID();
+	XMLElementNode *clipElem = new XMLElementNode("clipPath");
+	clipElem->addAttribute("id", XMLString("clip")+XMLString(newID));
+	if (!COMPUTE_CLIPPATHS_INTERSECTIONS && oldID)
+		clipElem->addAttribute("clip-path", XMLString("url(#clip")+XMLString(oldID)+")");
+
+	clipElem->append(pathElem);
+	_actions->appendToDefs(clipElem);
 }
 
 
 /** Clears current path */
 void PsSpecialHandler::newpath (vector<double> &p) {
-	_path.newpath();
+	bool drawing = (p[0] > 0);
+	if (!drawing || !_clipStack.clippathLoaded()) {
+		_path.clear();
+		_clipStack.setClippathLoaded(false);
+	}
 }
 
 
@@ -786,7 +846,7 @@ void PsSpecialHandler::executed () {
 
 ////////////////////////////////////////////
 
-void PsSpecialHandler::ClippingStack::push () {
+void PsSpecialHandler::ClippingStack::pushEmptyPath () {
 	if (!_stack.empty())
 		_stack.push(Entry(0, -1));
 }
@@ -840,6 +900,24 @@ const PsSpecialHandler::Path* PsSpecialHandler::ClippingStack::top () const {
 }
 
 
+PsSpecialHandler::Path* PsSpecialHandler::ClippingStack::getPath (size_t id) {
+	return (id > 0 && id <= _paths.size()) ? &_paths[id-1] : 0;
+}
+
+
+/** Returns true if the clipping path was loaded into the graphics path (via PS operator 'clippath') */
+bool PsSpecialHandler::ClippingStack::clippathLoaded () const {
+	return !_stack.empty() && _stack.top().cpathLoaded;
+}
+
+
+void PsSpecialHandler::ClippingStack::setClippathLoaded (bool loaded) {
+	if (_stack.empty())
+		return;
+	_stack.top().cpathLoaded = loaded;
+}
+
+
 /** Pops all elements from the stack. */
 void PsSpecialHandler::ClippingStack::clear() {
 	_paths.clear();
@@ -871,4 +949,3 @@ const char** PsSpecialHandler::prefixes () const {
 	static const char *pfx[] = {"header=", "psfile=", "PSfile=", "ps:", "ps::", "!", "\"", 0};
 	return pfx;
 }
-
