@@ -53,19 +53,21 @@ struct SpecialActions
 	virtual BoundingBox& bbox (const std::string &name, bool reset=false) =0;
 	virtual void embed (const BoundingBox &bbox) =0;
 	virtual void embed (const DPair &p, double r=0) =0;
+	virtual unsigned getCurrentPageNumber () const =0;
+	virtual std::string getSVGFilename (unsigned pageno) const =0;
 	virtual void progress (const char *id) {}
-	virtual int getDVIStackDepth () const {return 0;}
+	virtual int getDVIStackDepth () const  {return 0;}
 };
 
 
-class SpecialEmptyActions : public SpecialActions
+class EmptySpecialActions : public SpecialActions
 {
 	public:
 		double getX () const {return 0;}
 		double getY () const {return 0;}
 		void setX (double x) {}
 		void setY (double y) {}
-   	void finishLine ()  {}
+		void finishLine ()  {}
 		void setColor (const Color &color) {}
 		void setBgColor (const Color &color) {}
 		Color getColor () const {return Color::BLACK;}
@@ -81,11 +83,12 @@ class SpecialEmptyActions : public SpecialActions
 		BoundingBox& bbox (const std::string &name, bool reset=false) {return _bbox;}
 		void embed (const BoundingBox &bbox) {}
 		void embed (const DPair &p, double r=0) {}
+		unsigned getCurrentPageNumber() const {return 0;}
+		std::string getSVGFilename (unsigned pageno) const {return "";}
 
 	private:
 		BoundingBox _bbox;
 		Matrix _matrix;
 };
 
-
 #endif
