@@ -35,6 +35,12 @@ SpecialManager::~SpecialManager () {
 }
 
 
+SpecialManager& SpecialManager::instance() {
+	static SpecialManager sm;
+	return sm;
+}
+
+
 /** Remove all registered handlers. */
 void SpecialManager::unregisterHandlers () {
 	FORALL(_pool, vector<SpecialHandler*>::iterator, it)
@@ -55,6 +61,8 @@ void SpecialManager::registerHandler (SpecialHandler *handler) {
 		_pool.push_back(handler);
 		for (const char **p=handler->prefixes(); *p; ++p)
 			_handlers[*p] = handler;
+		if (DVIPreprocessingListener *listener = dynamic_cast<DVIPreprocessingListener*>(handler))
+			_preprocListeners.push_back(listener);
 		if (DVIEndPageListener *listener = dynamic_cast<DVIEndPageListener*>(handler))
 			_endPageListeners.push_back(listener);
 		if (DVIPositionListener *listener = dynamic_cast<DVIPositionListener*>(handler))
@@ -111,32 +119,35 @@ static string extract_prefix (istream &is) {
 }
 
 
+void SpecialManager::preprocess (const string &special, SpecialActions *actions) const {
+	istringstream iss(special);
+	string prefix = extract_prefix(iss);
+	if (SpecialHandler *handler = findHandler(prefix))
+		handler->preprocess(prefix.c_str(), iss, actions);
+}
+
+
 /** Executes a special command.
  *  @param[in] special the special expression
  *  @param[in] dvi2bp factor to convert DVI units to PS points
  *  @param[in] actions actions the special handlers can perform
- *  @param[in] listener object that wants to be notified about the processing state
  *  @return true if the special could be processed successfully
  *  @throw SpecialException in case of errors during special processing */
-bool SpecialManager::process (const string &special, double dvi2bp, SpecialActions *actions, Listener *listener) const {
+bool SpecialManager::process (const string &special, double dvi2bp, SpecialActions *actions) const {
 	istringstream iss(special);
 	string prefix = extract_prefix(iss);
 	bool success=false;
 	if (SpecialHandler *handler = findHandler(prefix)) {
-		if (listener)
-			listener->beginSpecial(prefix.c_str());
 		handler->setDviScaleFactor(dvi2bp);
 		success = handler->process(prefix.c_str(), iss, actions);
-		if (listener)
-			listener->endSpecial(prefix.c_str());
 	}
 	return success;
 }
 
 
-void SpecialManager::leavePSHeaderSection () const {
-	if (PsSpecialHandler *pshandler = dynamic_cast<PsSpecialHandler*>(findHandler("!")))
-		pshandler->enterBodySection();
+void SpecialManager::notifyPreprocessingFinished () const {
+	FORALL(_preprocListeners, vector<DVIPreprocessingListener*>::const_iterator, it)
+		(*it)->dviPreprocessingFinished();
 }
 
 
