@@ -31,40 +31,34 @@ struct SpecialActions;
 
 class SpecialManager
 {
-	public:
-		struct Listener
-		{
-			virtual ~Listener () {}
-			virtual void beginSpecial (const char *prefix)=0;
-			virtual void endSpecial (const char *prefix)=0;
-		};
-
 	private:
 		typedef std::vector<SpecialHandler*> HandlerPool;
 		typedef std::map<std::string,SpecialHandler*> HandlerMap;
 		typedef HandlerMap::iterator Iterator;
 		typedef HandlerMap::const_iterator ConstIterator;
 
-   public:
-		SpecialManager () {}
-      virtual ~SpecialManager ();
+	public:
+		~SpecialManager ();
+		static SpecialManager& instance ();
 		void registerHandler (SpecialHandler *handler);
 		void registerHandlers (SpecialHandler **handlers, const char *ignorelist);
 		void unregisterHandlers ();
-		bool process (const std::string &special, double dvi2bp, SpecialActions *actions, Listener *listener=0) const;
+		void preprocess (const std::string &special, SpecialActions *actions) const;
+		bool process (const std::string &special, double dvi2bp, SpecialActions *actions) const;
+		void notifyPreprocessingFinished () const;
 		void notifyEndPage (unsigned pageno) const;
 		void notifyPositionChange (double x, double y) const;
 		void writeHandlerInfo (std::ostream &os) const;
-		void leavePSHeaderSection () const;
 
 	protected:
+		SpecialManager () {}
 		SpecialManager (const SpecialManager &) {}
-		void operator = (const SpecialManager &) {}
 		SpecialHandler* findHandler (const std::string &prefix) const;
 
-   private:
+	private:
 		HandlerPool _pool;     ///< stores pointers to all handlers
 		HandlerMap _handlers;  ///< pointers to handlers for corresponding prefixes
+		std::vector<DVIPreprocessingListener*> _preprocListeners;
 		std::vector<DVIEndPageListener*> _endPageListeners;
 		std::vector<DVIPositionListener*> _positionListeners;
 };
