@@ -2,7 +2,7 @@
 ** StreamWriter.h                                                       **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef STREAMWRITER_H
-#define STREAMWRITER_H
+#ifndef DVISVGM_STREAMWRITER_H
+#define DVISVGM_STREAMWRITER_H
 
 #include <ostream>
 #include "types.h"
@@ -29,9 +29,9 @@ class CRC32;
 
 class StreamWriter
 {
-   public:
-      StreamWriter (std::ostream &os) : _os(os) {}
-      virtual ~StreamWriter () {}
+	public:
+		StreamWriter (std::ostream &os) : _os(os) {}
+		virtual ~StreamWriter () {}
 		void writeUnsigned (UInt32 val, int n);
 		void writeSigned (Int32 val, int n);
 		void writeString (const std::string &str, bool finalZero=false);
@@ -39,7 +39,7 @@ class StreamWriter
 		void writeSigned (Int32 val, int n, CRC32 &crc32);
 		void writeString (const std::string &str, CRC32 &crc32, bool finalZero=false);
 
-   private:
+	private:
 		std::ostream &_os;
 };
 
