@@ -2,7 +2,7 @@
 ** System.h                                                             **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,12 +18,12 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef SYSTEM_H
-#define SYSTEM_H
+#ifndef DVISVGM_SYSTEM_H
+#define DVISVGM_SYSTEM_H
 
 namespace System
 {
 	double time ();
-};
+}
 
 #endif
