@@ -33,8 +33,8 @@ struct XMLNode
 {
 	virtual ~XMLNode () {}
 	virtual XMLNode* clone () const =0;
+	virtual void clear () =0;
 	virtual std::ostream& write (std::ostream &os) const =0;
-	virtual bool emit (std::ostream &os, XMLNode *stopElement);
 	virtual void append (XMLNode *child) {}
 	virtual void prepend (XMLNode *child) {}
 };
@@ -45,31 +45,32 @@ class XMLElementNode : public XMLNode
 	typedef std::map<std::string,std::string> AttribMap;
 	typedef std::list<XMLNode*> ChildList;
 	public:
-      XMLElementNode (const std::string &name);
+		XMLElementNode (const std::string &name);
 		XMLElementNode (const XMLElementNode &node);
 		~XMLElementNode ();
 		XMLNode* clone () const {return new XMLElementNode(*this);}
+		void clear ();
 		void addAttribute (const std::string &name, const std::string &value);
 		void addAttribute (const std::string &name, double value);
 		void append (XMLNode *child);
 		void append (const std::string &str);
 		void prepend (XMLNode *child);
+		void remove (XMLNode *child)                 {_children.remove(child);}
 		bool insertAfter (XMLNode *child, XMLNode *sibling);
 		bool insertBefore (XMLNode *child, XMLNode *sibling);
 		bool hasAttribute (const std::string &name) const;
 		const char* getAttributeValue (const std::string &name) const;
-		bool findDescendants (const char *name, const char *attr_name, std::vector<XMLElementNode*> &descendants);
+		bool getDescendants (const char *name, const char *attr_name, std::vector<XMLElementNode*> &descendants) const;
+		XMLElementNode* getFirstDescendant (const char *name, const char *attrName, const char *attrValue) const;
 		std::ostream& write (std::ostream &os) const;
-		bool emit (std::ostream &os, XMLNode *stopElement);
 		bool empty () const                          {return _children.empty();}
-      const std::list<XMLNode*>& children () const {return _children;}
-      const std::string& getName () const          {return _name;}
+		const std::list<XMLNode*>& children () const {return _children;}
+		const std::string& getName () const          {return _name;}
 
 	private:
 		std::string _name;     // element name (<name a1="v1" .. an="vn">...</name>)
 		AttribMap _attributes;
 		ChildList _children;   // child nodes
-		bool _emitted;         // true if node has been (partly) emitted
 };
 
 
@@ -78,11 +79,13 @@ class XMLTextNode : public XMLNode
 	public:
 		XMLTextNode (const std::string &str) : _text(str) {}
 		XMLNode* clone () const {return new XMLTextNode(*this);}
+		void clear ()           {_text.clear();}
 		void append (XMLNode *node);
 		void append (XMLTextNode *node);
 		void append (const std::string &str);
 		void prepend (XMLNode *child);
 		std::ostream& write (std::ostream &os) const {return os << _text;}
+		const std::string& getText () const {return _text;}
 
 	private:
 		std::string _text;
@@ -94,6 +97,7 @@ class XMLCommentNode : public XMLNode
 	public:
 		XMLCommentNode (const std::string &str) : _text(str) {}
 		XMLNode* clone () const {return new XMLCommentNode(*this);}
+		void clear ()           {_text.clear();}
 		std::ostream& write (std::ostream &os) const {return os << "<!--" << _text << "-->\n";}
 
 	private:
@@ -108,15 +112,14 @@ class XMLDeclarationNode : public XMLNode
 		XMLDeclarationNode (const XMLDeclarationNode &node);
 		~XMLDeclarationNode ();
 		XMLNode* clone () const {return new XMLDeclarationNode(*this);}
+		void clear ();
 		void append (XMLNode *child);
 		std::ostream& write (std::ostream &os) const;
-		bool emit (std::ostream &os, XMLNode *stopElement);
 
 	private:
 		std::string _name;
 		std::string _params;
 		std::list<XMLDeclarationNode*> _children;
-		bool _emitted;
 };
 
 
@@ -125,6 +128,7 @@ class XMLCDataNode : public XMLNode
 	public:
 		XMLCDataNode (const std::string &d) : _data(d) {}
 		XMLNode* clone () const {return new XMLCDataNode(*this);}
+		void clear ()           {return _data.clear();}
 		std::ostream& write (std::ostream &os) const;
 
 	private:
