@@ -2,7 +2,7 @@
 ** macros.h                                                             **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef MACROS_H
-#define MACROS_H
+#ifndef DVISVGM_MACROS_H
+#define DVISVGM_MACROS_H
 
 #define FORALL(container, itertype, itervar) \
 	for (itertype itervar=(container).begin(); itervar != (container).end(); ++itervar)
