@@ -1,6 +1,6 @@
 /* otftotfm.cc -- driver for translating OpenType fonts to TeX metrics
  *
- * Copyright (c) 2003-2012 Eddie Kohler
+ * Copyright (c) 2003-2014 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -63,6 +63,9 @@
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
+#ifdef _MSC_VER
+# include <io.h>
+#endif
 
 using namespace Efont;
 
@@ -106,6 +109,7 @@ using namespace Efont;
 #define FIXED_PITCH_OPT		342
 #define ITALIC_ANGLE_OPT	343
 #define PROPORTIONAL_WIDTH_OPT	344
+#define X_HEIGHT_OPT		345
 
 #define AUTOMATIC_OPT		350
 #define FONT_NAME_OPT		351
@@ -188,6 +192,7 @@ static Clp_Option options[] = {
     { "fixed-width", 0, FIXED_PITCH_OPT, 0, Clp_Negate },
     { "proportional-width", 0, PROPORTIONAL_WIDTH_OPT, 0, Clp_Negate },
     { "italic-angle", 0, ITALIC_ANGLE_OPT, Clp_ValDouble, 0 },
+    { "x-height", 0, X_HEIGHT_OPT, Clp_ValString, 0 },
 
     { "pl", 'p', PL_OPT, 0, 0 },
     { "virtual", 0, VIRTUAL_OPT, 0, Clp_Negate },
@@ -275,6 +280,8 @@ static bool override_is_fixed_pitch = false;
 static bool is_fixed_pitch;
 static bool override_italic_angle = false;
 static double italic_angle;
+static int override_x_height = FontInfo::x_height_auto;
+static double x_height;
 
 static String out_encoding_file;
 static String out_encoding_name;
@@ -335,6 +342,7 @@ Font feature and transformation options:\n\
       --design-size=SIZE       Set font design size to SIZE.\n\
       --fixed-width            Set fixed width (no space stretch).\n\
       --italic-angle=ANGLE     Set font italic angle (for positioning accents).\n\
+      --x-height=AMT           Set x-height to AMT units.\n\
 \n");
     uerrh.message("\
 Encoding options:\n\
@@ -511,29 +519,51 @@ lig_context_str(int ctx)
 
 static double max_printed_real;
 
-static void
-fprint_real(FILE *f, const char *prefix, double value, double du, const char *suffix = ")\n")
-{
-    if (du == 1.) {
-	fprintf(f, "%s R %g%s", prefix, value, suffix);
-	max_printed_real = std::max(max_printed_real, fabs(value));
-    } else {
-        fprintf(f, "%s R %.4f%s", prefix, value * du, suffix);
-	max_printed_real = std::max(max_printed_real, fabs(value * du));
+namespace {
+struct Printer {
+    Printer(FILE* f, unsigned design_units, unsigned units_per_em)
+        : f_(f), du_((double) design_units / units_per_em),
+          round_(design_units == 1000) {
     }
+    inline double transform(double value) const;
+    void print_transformed(const char* prefix, double value) const;
+    void print(const char* prefix, double value) const;
+    String render(double value) const;
+    FILE* f_;
+    double du_;
+    bool round_;
+};
+
+inline double Printer::transform(double value) const {
+    value *= du_;
+    if (round_)
+        value = ceil(value);
+    return value;
 }
 
-static String
-real_string(double value, double du)
-{
-    if (du == 1.)
+void Printer::print_transformed(const char* prefix, double value) const {
+    if (round_ || value == 0 || (value > 0.01 && value - floor(value) < 0.01))
+        fprintf(f_, "%s R %g)\n", prefix, value);
+    else
+        fprintf(f_, "%s R %.4f)\n", prefix, value);
+    max_printed_real = std::max(max_printed_real, fabs(value));
+}
+
+void Printer::print(const char* prefix, double value) const {
+    print_transformed(prefix, transform(value));
+}
+
+String Printer::render(double value) const {
+    value = transform(value);
+    if (round_ || value == 0 || (value > 0.01 && value - floor(value) < 0.01))
 	return String(value);
     else {
 	char buf[128];
-        sprintf(buf, "%.4f", value * du);
+        sprintf(buf, "%.4f", value);
         return String(buf);
     }
 }
+} // namespace
 
 double
 font_slant(const FontInfo &finfo)
@@ -591,8 +621,8 @@ output_pl(Metrics &metrics, const String &ps_name, int boundary_char,
 	font_xform.scale(extend, 1);
     if (slant)
 	font_xform.shear(slant);
-    int bounds[4], width;
-    double du = (design_units == metrics.units_per_em() ? 1. : design_units / (double) metrics.units_per_em());
+    double bounds[4], width;
+    Printer pr(f, design_units, metrics.units_per_em());
 
     double actual_slant = font_slant(finfo);
     if (actual_slant)
@@ -601,24 +631,24 @@ output_pl(Metrics &metrics, const String &ps_name, int boundary_char,
     if (char_bounds(bounds, width, finfo, font_xform, ' ')) {
 	// advance space width by letterspacing, scale by space_factor
 	double space_width = (width + (vpl ? letterspace : 0)) * space_factor;
-	fprint_real(f, "   (SPACE", space_width, du);
+	pr.print("   (SPACE", space_width);
 	if (finfo.is_fixed_pitch()) {
 	    // fixed-pitch: no space stretch or shrink
-	    fprint_real(f, "   (STRETCH", 0, du);
-	    fprint_real(f, "   (SHRINK", 0, du);
-	    fprint_real(f, "   (EXTRASPACE", space_width, du);
+	    pr.print("   (STRETCH", 0);
+	    pr.print("   (SHRINK", 0);
+	    pr.print("   (EXTRASPACE", space_width);
 	} else {
-	    fprint_real(f, "   (STRETCH", space_width / 2., du);
-	    fprint_real(f, "   (SHRINK", space_width / 3., du);
-	    fprint_real(f, "   (EXTRASPACE", space_width / 6., du);
+	    pr.print("   (STRETCH", space_width / 2.);
+	    pr.print("   (SHRINK", space_width / 3.);
+	    pr.print("   (EXTRASPACE", space_width / 6.);
 	}
     }
 
-    int xheight = font_x_height(finfo, font_xform);
-    if (xheight * du < 1000)
-	fprint_real(f, "   (XHEIGHT", xheight, du);
+    double x_height = finfo.x_height(font_xform);
+    if (x_height < finfo.units_per_em())
+	pr.print("   (XHEIGHT", x_height);
 
-    fprint_real(f, "   (QUAD", finfo.units_per_em(), du);
+    pr.print("   (QUAD", finfo.units_per_em());
     fprintf(f, "   )\n");
 
     if (boundary_char >= 0)
@@ -727,7 +757,7 @@ output_pl(Metrics &metrics, const String &ps_name, int boundary_char,
 			double this_kern = kern_amt[k2 - kern_code2.begin()];
 			if (fabs(this_kern) >= minimum_kern)
 			    kern_sa << "   (KRN " << glyph_ids[*k2]
-				    << " R " << real_string(this_kern, du)
+				    << " R " << pr.render(this_kern)
 				    << ')' << glyph_comments[*k2] << '\n';
 		    }
 		if (kern_sa) {
@@ -776,9 +806,9 @@ output_pl(Metrics &metrics, const String &ps_name, int boundary_char,
 		      if (vpl)
 			  boundser.translate(s->x + x, s->y + y);
 		      if (s->x + x)
-			  sa << "      (MOVERIGHT R " << real_string(s->x + x, du) << ")\n";
+			  sa << "      (MOVERIGHT R " << pr.render(s->x + x) << ")\n";
 		      if (s->y + y)
-			  sa << "      (MOVEUP R " << real_string(s->y + y, du) << ")\n";
+			  sa << "      (MOVEUP R " << pr.render(s->y + y) << ")\n";
 		      break;
 		  }
 
@@ -788,7 +818,7 @@ output_pl(Metrics &metrics, const String &ps_name, int boundary_char,
 			boundser.mark(Point(s->x, s->y));
 			boundser.translate(s->x, 0);
 		    }
-		    sa << "      (SETRULE R " << real_string(s->y, du) << " R " << real_string(s->x, du) << ")\n";
+		    sa << "      (SETRULE R " << pr.render(s->y) << " R " << pr.render(s->x) << ")\n";
 		    break;
 
 		  case Setting::FONT:
@@ -838,13 +868,13 @@ output_pl(Metrics &metrics, const String &ps_name, int boundary_char,
 
 	    // output information
 	    boundser.output(bounds, width);
-	    fprint_real(f, "   (CHARWD", width, du);
+	    pr.print("   (CHARWD", width);
 	    if (bounds[3] > 0)
-		fprint_real(f, "   (CHARHT", bounds[3], du);
+		pr.print("   (CHARHT", bounds[3]);
 	    if (bounds[1] < 0)
-		fprint_real(f, "   (CHARDP", -bounds[1], du);
+		pr.print("   (CHARDP", -bounds[1]);
 	    if (bounds[2] > width)
-		fprint_real(f, "   (CHARIC", bounds[2] - width, du);
+		pr.print_transformed("   (CHARIC", pr.transform(bounds[2]) - pr.transform(width));
 	    if (vpl && (settings.size() > 1 || settings[0].op != Setting::SHOW))
 		fprintf(f, "   (MAP\n%s      )\n", sa.c_str());
 	    fprintf(f, "   )\n");
@@ -975,7 +1005,7 @@ write_encoding_file(String &filename, const String &encoding_name,
 
     // append old encodings
     int pos1 = old_encodings.find_left("\n%%");
-    while (pos1 < old_encodings.length()) {
+    while (pos1 >= 0 && pos1 < old_encodings.length()) {
 	int pos2 = old_encodings.find_left("\n%%", pos1 + 1);
 	if (pos2 < 0)
 	    pos2 = old_encodings.length();
@@ -1464,22 +1494,24 @@ do_math_spacing(Metrics &metrics, const FontInfo &finfo,
 	font_xform.shear(slant);
     CharstringBounds boundser(font_xform);
 
-    int x_height = font_x_height(finfo, font_xform);
+    double x_height = finfo.x_height(font_xform);
     double slant = font_slant(finfo);
     int boundary_char = dvipsenc.boundary_char();
 
-    int bounds[4], width;
+    double bounds[4], width;
 
     for (int code = 0; code < metrics.encoding_size(); code++)
 	if (metrics.was_base_glyph(code) && code != boundary_char
 	    && char_bounds(bounds, width, finfo, font_xform, code)) {
-	    int left_sb = (bounds[0] < 0 ? -bounds[0] : 0);
+	    int left_sb = (bounds[0] < 0 ? (int) ceil(-bounds[0]) : 0);
 	    metrics.add_single_positioning(code, left_sb, 0, left_sb);
 
 	    if (skew_char >= 0 && code < 256) {
-		double virtual_height = (bounds[3] > x_height ? bounds[3] :x_height) - 0.5 * x_height;
-		int right_sb = (bounds[2] > width ? bounds[2] - width : 0);
-		int skew = (int)(slant * virtual_height + left_sb - right_sb);
+		double virtual_height =
+                    (bounds[3] > x_height ? bounds[3] : x_height)
+                    - 0.5 * x_height;
+		double right_sb = (bounds[2] > width ? bounds[2] - width : 0);
+		int skew = (int) (slant * virtual_height + left_sb - right_sb);
 		metrics.add_kern(code, skew_char, skew);
 	    }
 	}
@@ -1499,6 +1531,8 @@ do_file(const String &otf_filename, const OpenType::Font &otf,
 	finfo.set_is_fixed_pitch(is_fixed_pitch);
     if (override_italic_angle)
 	finfo.set_italic_angle(italic_angle);
+    if (override_x_height != FontInfo::x_height_auto)
+        finfo.set_x_height(override_x_height, x_height);
 
     // save glyph names
     Vector<PermString> glyph_names;
@@ -1739,11 +1773,6 @@ parse_base_encodings(const String &filename, ErrorHandler *errh)
     }
 }
 
-#ifdef W32TEX
-#include <windows.h>
-HINSTANCE Hinstance;
-#endif
-
 int
 main(int argc, char *argv[])
 {
@@ -1755,15 +1784,6 @@ main(int argc, char *argv[])
     Clp_AddType(clp, CHAR_OPTTYPE, 0, clp_parse_char, 0);
     program_name = Clp_ProgramName(clp);
 #if HAVE_KPATHSEA
-#ifdef W32TEX
-    Hinstance = LoadLibrary(KPSEDLLNAME);
-    if(Hinstance == NULL)
-	Hinstance = LoadLibrary(KPSEOLDDLLNAME);
-    if(Hinstance == NULL) {
-	fprintf(stderr, "I cannot load Kpathsea dynamic library.\n");
-	return 100;
-    }
-#endif
     kpsei_init(argv[0], "lcdftools");
 #endif
 #ifdef HAVE_CTIME
@@ -1780,6 +1800,7 @@ main(int argc, char *argv[])
     const char *input_file = 0;
     Vector<String> glyphlist_files;
     bool literal_encoding = false;
+    bool have_encoding_file = false;
     Vector<String> ligkern;
     Vector<String> pos;
     Vector<String> unicoding;
@@ -1864,12 +1885,14 @@ main(int argc, char *argv[])
 	    if (encoding_file)
 		usage_error(errh, "encoding specified twice");
 	    encoding_file = clp->vstr;
+            have_encoding_file = true;
 	    break;
 
 	  case LITERAL_ENCODING_OPT:
 	    if (encoding_file)
 		usage_error(errh, "encoding specified twice");
 	    encoding_file = clp->vstr;
+            have_encoding_file = true;
 	    literal_encoding = true;
 	    break;
 
@@ -2129,25 +2152,34 @@ main(int argc, char *argv[])
 #endif
 	    break;
 
+        case X_HEIGHT_OPT: {
+            char* ends;
+            if (strcmp(clp->vstr, "auto") == 0)
+                override_x_height = FontInfo::x_height_auto;
+            else if (strcmp(clp->vstr, "x") == 0)
+                override_x_height = FontInfo::x_height_x;
+            else if (strcmp(clp->vstr, "font") == 0
+                     || strcmp(clp->vstr, "os/2") == 0)
+                override_x_height = FontInfo::x_height_os2;
+            else if ((x_height = strtod(clp->vstr, &ends)) >= 0
+                     && *ends == 0 && *clp->vstr != 0)
+                override_x_height = FontInfo::x_height_explicit;
+            else
+		usage_error(errh, "bad --x-height option");
+            break;
+        }
+
 	  case VERSION_OPT:
 	    printf("otftotfm (LCDF typetools) %s\n", VERSION);
-	    printf("Copyright (C) 2002-2012 Eddie Kohler\n\
+	    printf("Copyright (C) 2002-2014 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
-#ifdef W32TEX
-	    if(Hinstance != NULL)
-		FreeLibrary(Hinstance);
-#endif
 	    exit(0);
 	    break;
 
 	  case HELP_OPT:
 	    usage();
-#ifdef W32TEX
-	    if(Hinstance != NULL)
-		FreeLibrary(Hinstance);
-#endif
 	    exit(0);
 	    break;
 
@@ -2181,10 +2213,7 @@ particular purpose.\n");
     // set up file names
     if (!input_file)
 	usage_error(errh, "no font filename provided");
-    if (!encoding_file) {
-	errh->warning("no encoding provided");
-	errh->message("(Use %<-e ENCODING%> to choose an encoding. %<-e texnansx%> often works,\nor say %<-e -%> to turn off this warning.)");
-    } else if (encoding_file == "-")
+    if (encoding_file == "-")
 	encoding_file = "";
 
     // set up feature filters
@@ -2203,13 +2232,8 @@ particular purpose.\n");
     try {
 	// read font
 	otf_data = read_file(input_file, errh);
-	if (errh->nerrors()) {
-#ifdef W32TEX
-	    if(Hinstance != NULL)
-		FreeLibrary(Hinstance);
-#endif
+	if (errh->nerrors())
 	    exit(1);
-	}
 
 	LandmarkErrorHandler cerrh(errh, printable_filename(input_file));
 	BailErrorHandler bail_errh(&cerrh);
@@ -2262,8 +2286,18 @@ particular purpose.\n");
 	    else
 		errh->fatal("encoding %<%s%> not found", encoding_file.c_str());
 	} else {
+            String cff_data(otf.table("CFF"));
+            if (!cff_data)
+                errh->error("explicit encoding required for TrueType fonts");
+            else if (!have_encoding_file)
+                errh->warning("no encoding provided");
+            if (!cff_data || !have_encoding_file)
+                errh->message("(Use %<-e ENCODING%> to choose an encoding. %<-e texnansx%> often works,\nor say %<-e -%> to turn off this warning.)");
+            if (!cff_data)
+                exit(1);
+
 	    // use encoding from font
-	    Cff cff(otf.table("CFF"), &bail_errh);
+	    Cff cff(cff_data, otf.units_per_em(), &bail_errh);
 	    Cff::FontParent *font = cff.font(PermString(), &bail_errh);
 	    assert(cff.ok() && font->ok());
 	    if (Type1Encoding *t1e = font->type1_encoding()) {
@@ -2298,10 +2332,6 @@ particular purpose.\n");
     } catch (OpenType::Error e) {
 	errh->error("unhandled exception %<%s%>", e.description.c_str());
     }
-#ifdef W32TEX
-    if(Hinstance != NULL)
-	FreeLibrary(Hinstance);
-#endif
 
     return (errh->nerrors() == 0 ? 0 : 1);
 }
