@@ -35,6 +35,7 @@
 #endif
  
 #ifdef KPATHSEA
+#include <kpathsea/config.h>
 #include <kpathsea/c-fopen.h>
 #include <kpathsea/getopt.h>
 #else
@@ -132,9 +133,10 @@ i32	OutputFontIndex;	/* current (new) index in ouput */
 
 struct	pagelist *PageList;	/* the list of allowed pages */
 
-const char	*DVIFileName;		/* name of input DVI file */
-FILE	*inf;			/* the input file itself */
-FILE	*outf;			/* the output DVI file */
+const char	*DVIFileName;	/* name of input DVI file */
+FILE		*inf;		/* the input file itself */
+const char	*outname;	/* name of output DVI file */
+FILE		*outf;		/* the output DVI file */
 
 long	StartOfLastPage;	/* The file position just before we started
 				   the last page (this is later written to
@@ -162,12 +164,7 @@ static int evenodd(char *);
 static int ParsePages(char *);
 static void HandleDVIFile(void);
 static void PutFontSelector(i32);
-
-#ifdef _AMIGA
-#define bcmp(s1, s2, len) memcmp(s1, s2, len)
-#define bzero(s, len) memset(s, '\0', len)
-#define index(s, c) strchr(s, c)
-#endif /* _AMIGA */
+static void WritePreAmble(void);
 
 #ifndef KPATHSEA
 char	*malloc(), *realloc();
@@ -287,6 +284,9 @@ BeginPage(void)
 	if ((UseThisPage = DesiredPageP()) == 0)
 		return;
 
+	if (NumberOfOutputPages == 0)
+		WritePreAmble();
+
 	putbyte(outf, DVI_BOP);
 	for (i = Count; i < &Count[10]; i++)
 		PutLong(outf, *i);
@@ -354,6 +354,9 @@ HandlePostAmble(void)
 {
 	register i32 c;
 
+	if (NumberOfOutputPages == 0)
+		return;
+
 	(void) GetLong(inf);	/* previous page pointer */
 	if (GetLong(inf) != Numerator)
 		GripeMismatchedValue("numerator");
@@ -446,12 +449,18 @@ WriteFont(struct fontinfo *fi)
 }
 
 /*
- * Handle the preamble.  Someday we should update the comment field.
+ * Array to hold the preamble comment.
+ */
+static char PreComment[256];
+
+/*
+ * Read the preamble.  Someday we should update the comment field.
  */
 static void
-HandlePreAmble(void)
+ReadPreAmble(void)
 {
 	register int n, c;
+	char *comment = PreComment;
 
 	c = getc(inf);
 	if (c == EOF)
@@ -464,17 +473,43 @@ HandlePreAmble(void)
 	Numerator = GetLong(inf);
 	Denominator = GetLong(inf);
 	DVIMag = GetLong(inf);
+	c = GetByte(inf);
+	n = UnSign8(c);
+	*comment++ = (char)c;
+	while (--n >= 0) {
+		c = GetByte(inf);
+		*comment++ = (char)c;
+	}
+}
+
+/*
+ * Open the output and write the preamble.
+ */
+static void
+WritePreAmble(void)
+{
+	register int n, c;
+	char *comment = PreComment;
+
+	if (outname == NULL) {
+		outf = stdout;
+		if (!isatty(fileno(outf)))
+		  SET_BINARY(fileno(outf));
+	} else if ((outf = fopen(outname, FOPEN_WBIN_MODE)) == 0)
+		error(1, -1, "cannot write %s", outname);
+
 	putbyte(outf, DVI_PRE);
 	putbyte(outf, DVI_VERSION);
 	PutLong(outf, Numerator);
 	PutLong(outf, Denominator);
 	PutLong(outf, DVIMag);
 
-	n = UnSign8(GetByte(inf));
+	c = *comment++;
+	n = UnSign8(c);
 	CurrentPosition = 15 + n;	/* well, almost */
 	putbyte(outf, n);
 	while (--n >= 0) {
-		c = GetByte(inf);
+		c = *comment++;
 		putbyte(outf, c);
 	}
 }
@@ -484,7 +519,6 @@ main(int argc, char **argv)
 {
 	register int c;
 	register char *s;
-	char *outname = NULL;
 
 	ProgName = *argv;
 	setbuf(stderr, serrbuf);
@@ -543,25 +577,25 @@ Usage: %s [-s] [-i infile] [-o outfile] pages [...] [infile [outfile]]\n",
 		  SET_BINARY(fileno(inf));
 	} else if ((inf = fopen(DVIFileName, FOPEN_RBIN_MODE)) == 0)
 		error(1, -1, "cannot read %s", DVIFileName);
-	if (outname == NULL) {
-		outf = stdout;
-		if (!isatty(fileno(outf)))
-		  SET_BINARY(fileno(outf));
-	} else if ((outf = fopen(outname, FOPEN_WBIN_MODE)) == 0)
-		error(1, -1, "cannot write %s", outname);
 
 	if ((FontFinder = SCreate(sizeof(struct fontinfo))) == 0)
 		error(1, 0, "cannot create font finder (out of memory?)");
 
 	StartOfLastPage = -1;
-	HandlePreAmble();
+	ReadPreAmble();
 	HandleDVIFile();
 	HandlePostAmble();
-	if (!SFlag)
-		(void) fprintf(stderr, "\nWrote %d page%s, %ld bytes\n",
-		    NumberOfOutputPages, NumberOfOutputPages == 1 ? "" : "s",
-		    (long)CurrentPosition);
-	return 0;
+	if (NumberOfOutputPages > 0) {
+		if (!SFlag)
+			(void) fprintf(stderr, "\nWrote %d page%s, %ld bytes\n",
+			    NumberOfOutputPages, NumberOfOutputPages == 1 ? "" : "s",
+ 			    (long)CurrentPosition);
+		return 0;
+	} else {
+		if (!SFlag)
+			(void) fprintf(stderr, "Specified page may be out of range\n");
+		return 1;
+	}
 }
 
 static struct pagelist *
