@@ -73,7 +73,7 @@
 #define IS_upTeX 1
 #include <euptexdir/euptexextra.h>
 #else
-#define BANNER "This is TeX, Version 3.1415926"
+#define BANNER "This is TeX, Version 3.14159265"
 #define COPYRIGHT_HOLDER "D.E. Knuth"
 #define AUTHOR NULL
 #define PROGRAM_HELP TEXHELP
@@ -89,7 +89,7 @@
 #define edit_var "TEXEDIT"
 #endif /* TeX */
 #ifdef MF
-#define BANNER "This is Metafont, Version 2.718281"
+#define BANNER "This is Metafont, Version 2.7182818"
 #define COPYRIGHT_HOLDER "D.E. Knuth"
 #define AUTHOR NULL
 #define PROGRAM_HELP MFHELP
@@ -205,50 +205,37 @@ Isspace (char c)
 static char **cmdlist = NULL;
 
 static void 
-mk_shellcmdlist (const char *v)
+mk_shellcmdlist (char *v)
 {
   char **p;
-  const char *q, *r1;
-  char *r;
-  int  n;
+  char *q, *r;
+  size_t n;
 
   q = v;
-  n = 0;
+  n = 1;
 
 /* analyze the variable shell_escape_commands = foo,bar,...
    spaces before and after (,) are not allowed. */
 
-  while ((r1 = strchr (q, ',')) != 0) {
+  while ((r = strchr (q, ',')) != 0) {
     n++;
-    r1++;
-    q = r1;
+    q = r + 1;
   }
   if (*q)
     n++;
-  cmdlist = xmalloc ((n + 1) * sizeof (char *));
+  cmdlist = xmalloc (n * sizeof (char *));
   p = cmdlist;
   q = v;
   while ((r = strchr (q, ',')) != 0) {
     *r = '\0';
-    *p = xmalloc (strlen (q) + 1);
-    strcpy (*p, q);
-    *r = ',';
-    r++;
-    q = r;
-    p++;
+    *p++ = xstrdup (q);
+    q = r + 1;
   }
-  if (*q) {
-    *p = xmalloc (strlen (q) + 1);
-    strcpy (*p, q);
-    p++;
-    *p = NULL;
-  } else
-    *p = NULL;
+  if (*q)
+    *p++ = xstrdup (q);
+  *p = NULL;
 }
 
-/* Called from maininit.  Not static because also called from
-   luatexdir/lua/luainit.c.  */
-
 static void
 init_shell_escape (void)
 {
@@ -273,7 +260,7 @@ init_shell_escape (void)
     if (shellenabledp && restrictedshell == 1) {
       char *v2 = kpse_var_value ("shell_escape_commands");
       if (v2) {
-        mk_shellcmdlist ((const char *)v2);
+        mk_shellcmdlist (v2);
         free (v2);
       }
     }
@@ -452,9 +439,17 @@ shell_cmd_is_allowed (const char *cmd, char **safecmd, char **cmdname)
     {
       char *p, *q, *r;
       p = *safecmd;
-      if (!(IS_DIR_SEP (p[0]) && IS_DIR_SEP (p[1])) &&
-          !(p[1] == ':' && IS_DIR_SEP (p[2]))) { 
-        p = (char *) kpse_var_value ("SELFAUTOLOC");
+      if (strlen (p) > 2 && p[1] == ':' && !IS_DIR_SEP (p[2])) {
+          q = xmalloc (strlen (p) + 2);
+          q[0] = p[0];
+          q[1] = p[1];
+          q[2] = '/';
+          q[3] = '\0';
+          strcat (q, (p + 2));
+          free (*safecmd);
+          *safecmd = q;
+      } else if (!IS_DIR_SEP (p[0]) && !(p[1] == ':' && IS_DIR_SEP (p[2]))) { 
+        p = kpse_var_value ("SELFAUTOLOC");
         if (p) {
           r = *safecmd;
           while (*r && !Isspace(*r))
@@ -491,7 +486,16 @@ shell_cmd_is_allowed (const char *cmd, char **safecmd, char **cmdname)
    1 if shell escapes are not restricted, hence any command is allowed.
    2 if shell escapes are restricted and CMD is allowed (possibly after
       quoting).  */
-   
+
+#ifdef WIN32
+#undef system
+#define system fsyscp_system
+#if ENABLE_PIPES
+#undef popen
+#define popen fsyscp_popen
+#endif /* ENABLE_PIPES */
+#endif /* WIN32 */
+
 int
 runsystem (const char *cmd)
 {
@@ -582,9 +586,6 @@ int argc;
 /* If the user overrides argv[0] with -progname.  */
 static const_string user_progname;
 
-/* The C version of what might wind up in DUMP_VAR.  */
-static const_string dump_name;
-
 /* The C version of the jobname, if given. */
 static const_string c_job_name;
 
@@ -621,6 +622,30 @@ texmf_yesno(const_string var)
 const char *ptexbanner = BANNER;
 #endif
 
+#ifdef WIN32
+/* forward declaration */
+static string
+normalize_quotes (const_string name, const_string mesg);
+#ifndef TeX
+int srcspecialsp = 0;
+#endif
+/* Support of 8.3-name convention. If *buffer == NULL, nothing is done. */
+static void change_to_long_name (char **buffer)
+{
+  if (*buffer) {
+    char inbuf[260];
+    char outbuf[260];
+
+    memset (outbuf, 0, 260);
+    strcpy (inbuf, *buffer);
+    if (GetLongPathName (inbuf, outbuf, 260)) {
+      *buffer = (char *)realloc(*buffer, strlen(outbuf) + 1);
+      strcpy (*buffer, outbuf);
+    }
+  }
+}
+#endif /* WIN32 */
+
 /* The entry point: set up for reading the command line, which will
    happen in `topenin', then call the main body.  */
 
@@ -628,7 +653,9 @@ void
 maininit (int ac, string *av)
 {
   string main_input_file;
-
+#if (IS_upTeX || defined(XeTeX)) && defined(WIN32)
+  string enc;
+#endif
   /* Save to pass along to topenin.  */
   argc = ac;
   argv = av;
@@ -653,13 +680,24 @@ maininit (int ac, string *av)
 
 #if IS_pTeX
   kpse_set_program_name (argv[0], NULL);
-  initdefaultkanji ();
+  initkanji ();
+#endif
+#if defined(XeTeX) && defined(WIN32)
+  kpse_set_program_name (argv[0], NULL);
+#endif
+#if (IS_upTeX || defined(XeTeX)) && defined(WIN32)
+  enc = kpse_var_value("command_line_encoding");
+  get_command_line_args_utf8(enc, &argc, &argv);
 #endif
 
   /* If the user says --help or --version, we need to notice early.  And
      since we want the --ini option, have to do it before getting into
      the web (which would read the base file, etc.).  */
+#if (IS_upTeX || defined(XeTeX)) && defined(WIN32)
+  parse_options (argc, argv);
+#else
   parse_options (ac, av);
+#endif
 
 #if IS_pTeX
   /* In pTeX and friends, texmf.cnf is not recorded in the case of --recorder,
@@ -679,7 +717,7 @@ maininit (int ac, string *av)
   /* Do this early so we can inspect kpse_invocation_name and
      kpse_program_name below, and because we have to do this before
      any path searching.  */
-#if IS_pTeX
+#if IS_pTeX || (defined(XeTeX) && defined(WIN32))
   if (user_progname)
     kpse_reset_program_name (user_progname);
 #else
@@ -696,6 +734,60 @@ maininit (int ac, string *av)
   /* Were we given a simple filename? */
   main_input_file = get_input_file_name();
 
+#ifdef WIN32
+  if (main_input_file == NULL) {
+    string name;
+#ifndef XeTeX
+    boolean quoted;
+#endif
+
+    name = argv[argc-1];
+    if (name && name[0] != '-' && name[0] != '&' && name[0] != '\\') {
+      if (strlen (name) > 2 && isalpha (name[0]) && name[1] == ':' &&
+          name[2] == '\\') {
+        string pp;
+        for (pp = name; *pp; pp++) {
+          if (IS_KANJI (pp))
+            pp++;
+          else if (*pp == '\\')
+            *pp = '/';
+        }
+      }
+#ifdef XeTeX
+      name = normalize_quotes(argv[argc-1], "argument");
+      main_input_file = kpse_find_file(argv[argc-1], INPUT_FORMAT, false);
+      if (!srcspecialsp) {
+        change_to_long_name (&main_input_file);
+        if (main_input_file)
+          name = normalize_quotes(main_input_file, "argument");
+      }
+      argv[argc-1] = name;
+#else
+      name = normalize_quotes(argv[argc-1], "argument");
+      quoted = (name[0] == '"');
+      if (quoted) {
+        /* Overwrite last quote and skip first quote. */
+        name[strlen(name)-1] = '\0';
+        name++;
+      }
+      main_input_file = kpse_find_file(name, INPUT_FORMAT, false);
+      if (!srcspecialsp)
+        change_to_long_name (&main_input_file);
+      if (quoted) {
+        /* Undo modifications */
+        name[strlen(name)] = '"';
+        name--;
+      }
+      if (!srcspecialsp) {
+        if (main_input_file)
+          name = normalize_quotes(main_input_file, "argument");
+      }
+      argv[argc-1] = name;
+#endif
+    }
+  }
+#endif /* WIN32 */
+
   /* Second chance to activate file:line:error style messages, this
      time from texmf.cnf. */
   if (filelineerrorstylep < 0) {
@@ -828,7 +920,7 @@ maininit (int ac, string *av)
    happen in `topenin', then call the main body.  */
 
 int
-#if defined(WIN32) && defined(DLLPROC)
+#if defined(WIN32) && !defined(__MINGW32__) && defined(DLLPROC)
 DLLPROC (int ac, string *av)
 #else
 main (int ac, string *av)
@@ -1411,6 +1503,10 @@ get_input_file_name (void)
     name = normalize_quotes(argv[optind], "argument");
 #ifdef XeTeX
     input_file_name = kpse_find_file(argv[optind], INPUT_FORMAT, false);
+#ifdef WIN32
+    if (!srcspecialsp)
+      change_to_long_name (&input_file_name);
+#endif
 #else
     quoted = (name[0] == '"');
     if (quoted) {
@@ -1419,11 +1515,21 @@ get_input_file_name (void)
         name++;
     }
     input_file_name = kpse_find_file(name, INPUT_FORMAT, false);
+#ifdef WIN32
+    if (!srcspecialsp)
+      change_to_long_name (&input_file_name);
+#endif
     if (quoted) {
         /* Undo modifications */
         name[strlen(name)] = '"';
         name--;
     }
+#endif
+#ifdef WIN32
+    if (!srcspecialsp) {
+      if (input_file_name)
+        name = normalize_quotes (input_file_name, "argument");
+    }
 #endif
     argv[optind] = name;
   }
@@ -1589,14 +1695,14 @@ parse_options (int argc, string *argv)
       /* Try to start up the other end if it's not already.  */
       if (!ipc_is_open ()) {
 #ifdef WIN32
-        if (spawnlp (_P_NOWAIT, IPC_SERVER_CMD, IPC_SERVER_CMD, NULL) != -1) {
+        if (_spawnlp (_P_NOWAIT, IPC_SERVER_CMD, IPC_SERVER_CMD, NULL) != -1) {
 #else
         if (system (IPC_SERVER_CMD) == 0) {
 #endif
           unsigned i;
           for (i = 0; i < 20 && !ipc_is_open (); i++) {
 #ifdef WIN32
-            Sleep (2000);
+            Sleep (100); /* 2000ms is too long for a simple w32 example */
 #else
             sleep (2);
 #endif
@@ -1857,12 +1963,7 @@ open_in_or_pipe (FILE **f_ptr, int filefmt, const_string fopen_mode)
       fname = xmalloc(strlen((const_string)(nameoffile+1))+1);
       strcpy(fname,(const_string)(nameoffile+1));
       recorder_record_input (fname + 1);
-#ifdef WIN32
-      /* Use binary mode on Windows.  */
-      *f_ptr = runpopen(fname+1,"rb");
-#else
       *f_ptr = runpopen(fname+1,"r");
-#endif
       free(fname);
       for (i=0; i<NUM_PIPES; i++) {
         if (pipes[i]==NULL) {
@@ -1871,7 +1972,7 @@ open_in_or_pipe (FILE **f_ptr, int filefmt, const_string fopen_mode)
         }
       }
       if (*f_ptr)
-        setvbuf (*f_ptr,NULL,_IOLBF,0);
+        setvbuf (*f_ptr,NULL,_IONBF,0);
 #ifdef WIN32
       Poptr = *f_ptr;
 #endif
@@ -1882,6 +1983,48 @@ open_in_or_pipe (FILE **f_ptr, int filefmt, const_string fopen_mode)
     return open_input(f_ptr,filefmt,fopen_mode) ;
 }
 
+#ifdef XeTeX
+boolean
+u_open_in_or_pipe(unicodefile* f, integer filefmt, const_string fopen_mode, integer mode, integer encodingData)
+{
+    string fname = NULL;
+    int i; /* iterator */
+
+    /* opening a read pipe is straightforward, only have to
+       skip past the pipe symbol in the file name. filename
+       quoting is assumed to happen elsewhere (it does :-)) */
+
+    if (shellenabledp && *(nameoffile+1) == '|') {
+      /* the user requested a pipe */
+      *f = malloc(sizeof(UFILE));
+      (*f)->encodingMode = (mode == AUTO) ? UTF8 : mode;
+      (*f)->conversionData = 0;
+      (*f)->savedChar = -1;
+      (*f)->skipNextLF = 0;
+      (*f)->f = NULL;
+      fname = xmalloc(strlen((const_string)(nameoffile+1))+1);
+      strcpy(fname,(const_string)(nameoffile+1));
+      recorder_record_input (fname + 1);
+      (*f)->f = runpopen(fname+1,"r");
+      free(fname);
+      for (i=0; i<NUM_PIPES; i++) {
+        if (pipes[i]==NULL) {
+          pipes[i] = (*f)->f;
+          break;
+        }
+      }
+      if ((*f)->f)
+        setvbuf ((*f)->f,NULL,_IONBF,0);
+#ifdef WIN32
+      Poptr = (*f)->f;
+#endif
+
+      return (*f)->f != NULL;
+    }
+
+    return u_open_in(f, filefmt, fopen_mode, mode, encodingData);
+}
+#endif
 
 boolean
 open_out_or_pipe (FILE **f_ptr, const_string fopen_mode)
@@ -1904,19 +2047,10 @@ open_out_or_pipe (FILE **f_ptr, const_string fopen_mode)
            is better to be prepared */
         if (STREQ((fname+strlen(fname)-4),".tex"))
           *(fname+strlen(fname)-4) = 0;
-#ifdef WIN32
-        /* Use binary mode on Windows.  */
-        *f_ptr = runpopen(fname+1,"wb");
-#else
         *f_ptr = runpopen(fname+1,"w");
-#endif
         *(fname+strlen(fname)) = '.';
       } else {
-#ifdef WIN32
-        *f_ptr = runpopen(fname+1,"wb");
-#else
         *f_ptr = runpopen(fname+1,"w");
-#endif
       }
       recorder_record_output (fname + 1);
       free(fname);
@@ -1929,7 +2063,7 @@ open_out_or_pipe (FILE **f_ptr, const_string fopen_mode)
       }
 
       if (*f_ptr)
-        setvbuf(*f_ptr,NULL,_IOLBF,0);
+        setvbuf(*f_ptr,NULL,_IONBF,0);
 
       return *f_ptr != NULL;
     }
@@ -2088,13 +2222,13 @@ input_line (FILE *f)
     if (position == 0L) {  /* Detect and skip Byte order marks.  */
       int k1 = getc (f);
 
-      if (k1 == EOF || k1 == '\r' || k1 == '\n')
-        fseek (f, -1L , SEEK_CUR);
+      if (k1 != 0xff && k1 != 0xfe && k1 != 0xef)
+        rewind (f);
       else {
         int k2 = getc (f);
 
-        if (k2 == EOF || k2 == '\r' || k2 == '\n')
-          fseek (f, -2L , SEEK_CUR);
+        if (k2 != 0xff && k2 != 0xfe && k2 != 0xbb)
+          rewind (f);
         else if ((k1 == 0xff && k2 == 0xfe) || /* UTF-16(LE) */
                  (k1 == 0xfe && k2 == 0xff))   /* UTF-16(BE) */
           ;
@@ -2104,7 +2238,7 @@ input_line (FILE *f)
           if (k1 == 0xef && k2 == 0xbb && k3 == 0xbf) /* UTF-8 */
             ;
           else
-            fseek (f, -3L, SEEK_CUR);
+            rewind (f);
         }
       }
     }
@@ -2471,7 +2605,12 @@ maketexstring(const_string s)
   UInt32 rval;
   const unsigned char *cp = (const unsigned char *)s;
 #endif
+#if defined(TeX)
+  if (s == NULL || *s == 0)
+    return getnullstr();
+#else
   assert (s != 0);
+#endif
   len = strlen(s);
   checkpoolpointer (poolptr, len); /* in the XeTeX case, this may be more than enough */
 #ifdef XeTeX
@@ -2657,7 +2796,312 @@ makesrcspecial (strnumber srcfilename, int lineno)
 
   return (oldpoolptr);
 }
-#endif
+
+/* pdfTeX routines also used for e-pTeX and e-upTeX */
+#if defined (pdfTeX) || defined (epTeX) || defined (eupTeX)
+
+#include <kpathsea/c-stat.h>
+
+#define check_nprintf(size_get, size_want) \
+    if ((unsigned)(size_get) >= (unsigned)(size_want)) \
+        pdftex_fail ("snprintf failed: file %s, line %d", __FILE__, __LINE__);
+#  define check_buf(size, buf_size)                         \
+    if ((unsigned)(size) > (unsigned)(buf_size))            \
+        pdftex_fail("buffer overflow at file %s, line %d", __FILE__,  __LINE__ )
+#  define xfree(p)            do { if (p != NULL) free(p); p = NULL; } while (0)
+#  define MAX_CSTRING_LEN     1024 * 1024
+
+#if !defined (pdfTeX)
+#  define PRINTF_BUF_SIZE     1024
+static char print_buf[PRINTF_BUF_SIZE];
+
+/* Helper for pdftex_fail. */
+static void safe_print(const char *str)
+{
+    const char *c;
+    for (c = str; *c; ++c)
+        print(*c);
+}
+/* pdftex_fail may be called when a buffer overflow has happened/is
+   happening, therefore may not call mktexstring.  However, with the
+   current implementation it appears that error messages are misleading,
+   possibly because pool overflows are detected too late.
+
+   The output format of this fuction must be the same as pdf_error in
+   pdftex.web! */
+__attribute__ ((noreturn, format(printf, 1, 2)))
+void pdftex_fail(const char *fmt, ...)
+{
+    va_list args;
+    va_start(args, fmt);
+    println();
+    safe_print("!error: ");
+    vsnprintf(print_buf, PRINTF_BUF_SIZE, fmt, args);
+    safe_print(print_buf);
+    va_end(args);
+    println();
+    safe_print(" ==> Fatal error occurred, output file will be damaged!");
+    println();
+    if (kpathsea_debug) {
+        safe_print("kpathsea_debug enabled, calling abort()...");
+        println();
+        abort();
+    } else {
+        exit(EXIT_FAILURE);
+    }
+}
+#endif /* not pdfTeX */
+
+static time_t start_time = 0;
+#define TIME_STR_SIZE 30
+char start_time_str[TIME_STR_SIZE];
+static char time_str[TIME_STR_SIZE];
+    /* minimum size for time_str is 24: "D:YYYYmmddHHMMSS+HH'MM'" */
+
+static void makepdftime(time_t t, char *time_str)
+{
+
+    struct tm lt, gmt;
+    size_t size;
+    int i, off, off_hours, off_mins;
+
+    /* get the time */
+    lt = *localtime(&t);
+    size = strftime(time_str, TIME_STR_SIZE, "D:%Y%m%d%H%M%S", &lt);
+    /* expected format: "YYYYmmddHHMMSS" */
+    if (size == 0) {
+        /* unexpected, contents of time_str is undefined */
+        time_str[0] = '\0';
+        return;
+    }
+
+    /* correction for seconds: %S can be in range 00..61,
+       the PDF reference expects 00..59,
+       therefore we map "60" and "61" to "59" */
+    if (time_str[14] == '6') {
+        time_str[14] = '5';
+        time_str[15] = '9';
+        time_str[16] = '\0';    /* for safety */
+    }
+
+    /* get the time zone offset */
+    gmt = *gmtime(&t);
+
+    /* this calculation method was found in exim's tod.c */
+    off = 60 * (lt.tm_hour - gmt.tm_hour) + lt.tm_min - gmt.tm_min;
+    if (lt.tm_year != gmt.tm_year) {
+        off += (lt.tm_year > gmt.tm_year) ? 1440 : -1440;
+    } else if (lt.tm_yday != gmt.tm_yday) {
+        off += (lt.tm_yday > gmt.tm_yday) ? 1440 : -1440;
+    }
+
+    if (off == 0) {
+        time_str[size++] = 'Z';
+        time_str[size] = 0;
+    } else {
+        off_hours = off / 60;
+        off_mins = abs(off - off_hours * 60);
+        i = snprintf(&time_str[size], 9, "%+03d'%02d'", off_hours, off_mins);
+        check_nprintf(i, 9);
+    }
+}
+
+void initstarttime(void)
+{
+    if (start_time == 0) {
+        start_time = time((time_t *) NULL);
+        makepdftime(start_time, start_time_str);
+    }
+}
+
+char *makecstring(integer s)
+{
+    static char *cstrbuf = NULL;
+    char *p;
+    static int allocsize;
+    int allocgrow, i, l = strstart[s + 1] - strstart[s];
+    check_buf(l + 1, MAX_CSTRING_LEN);
+    if (cstrbuf == NULL) {
+        allocsize = l + 1;
+        cstrbuf = xmallocarray(char, allocsize);
+    } else if (l + 1 > allocsize) {
+        allocgrow = allocsize * 0.2;
+        if (l + 1 - allocgrow > allocsize)
+            allocsize = l + 1;
+        else if (allocsize < MAX_CSTRING_LEN - allocgrow)
+            allocsize += allocgrow;
+        else
+            allocsize = MAX_CSTRING_LEN;
+        cstrbuf = xreallocarray(cstrbuf, char, allocsize);
+    }
+    p = cstrbuf;
+    for (i = 0; i < l; i++)
+        *p++ = strpool[i + strstart[s]];
+    *p = 0;
+    return cstrbuf;
+}
+
+/* makecfilename
+  input/ouput same as makecstring:
+    input: string number
+    output: C string with quotes removed.
+    That means, file names that are legal on some operation systems
+    cannot any more be used since pdfTeX version 1.30.4.
+*/
+char *makecfilename(integer s)
+{
+    char *name = makecstring(s);
+    char *p = name;
+    char *q = name;
+
+    while (*p) {
+        if (*p != '"')
+            *q++ = *p;
+        p++;
+    }
+    *q = '\0';
+    return name;
+}
+
+void getcreationdate(void)
+{
+    size_t len;
+    initstarttime();
+    /* put creation date on top of string pool and update poolptr */
+    len = strlen(start_time_str);
+
+    /* In e-pTeX, "init len => call initstarttime()" (as pdftexdir/utils.c)
+       yields  unintentional output. */
+
+    if ((unsigned) (poolptr + len) >= (unsigned) (poolsize)) {
+        poolptr = poolsize;
+        /* error by str_toks that calls str_room(1) */
+        return;
+    }
+
+    memcpy(&strpool[poolptr], start_time_str, len);
+    poolptr += len;
+}
+
+void getfilemoddate(integer s)
+{
+    struct stat file_data;
+
+    char *file_name = kpse_find_tex(makecfilename(s));
+    if (file_name == NULL) {
+        return;                 /* empty string */
+    }
+
+    recorder_record_input(file_name);
+    /* get file status */
+    if (stat(file_name, &file_data) == 0) {
+        size_t len;
+
+        makepdftime(file_data.st_mtime, time_str);
+        len = strlen(time_str);
+        if ((unsigned) (poolptr + len) >= (unsigned) (poolsize)) {
+            poolptr = poolsize;
+            /* error by str_toks that calls str_room(1) */
+        } else {
+            memcpy(&strpool[poolptr], time_str, len);
+            poolptr += len;
+        }
+    }
+    /* else { errno contains error code } */
+
+    xfree(file_name);
+}
+
+void getfilesize(integer s)
+{
+    struct stat file_data;
+    int i;
+
+    char *file_name = kpse_find_tex(makecfilename(s));
+    if (file_name == NULL) {
+        return;                 /* empty string */
+    }
+
+    recorder_record_input(file_name);
+    /* get file status */
+    if (stat(file_name, &file_data) == 0) {
+        size_t len;
+        char buf[20];
+
+        /* st_size has type off_t */
+        i = snprintf(buf, sizeof(buf),
+                     "%lu", (long unsigned int) file_data.st_size);
+        check_nprintf(i, sizeof(buf));
+        len = strlen(buf);
+        if ((unsigned) (poolptr + len) >= (unsigned) (poolsize)) {
+            poolptr = poolsize;
+            /* error by str_toks that calls str_room(1) */
+        } else {
+            memcpy(&strpool[poolptr], buf, len);
+            poolptr += len;
+        }
+    }
+    /* else { errno contains error code } */
+
+    xfree(file_name);
+}
+
+void getfiledump(integer s, int offset, int length)
+{
+    FILE *f;
+    int read, i;
+    poolpointer data_ptr;
+    poolpointer data_end;
+    char *file_name;
+
+    if (length == 0) {
+        /* empty result string */
+        return;
+    }
+
+    if (poolptr + 2 * length + 1 >= poolsize) {
+        /* no place for result */
+        poolptr = poolsize;
+        /* error by str_toks that calls str_room(1) */
+        return;
+    }
+
+    file_name = kpse_find_tex(makecfilename(s));
+    if (file_name == NULL) {
+        return;                 /* empty string */
+    }
+
+    /* read file data */
+    f = fopen(file_name, FOPEN_RBIN_MODE);
+    if (f == NULL) {
+        xfree(file_name);
+        return;
+    }
+    recorder_record_input(file_name);
+    if (fseek(f, offset, SEEK_SET) != 0) {
+        xfree(file_name);
+        return;
+    }
+    /* there is enough space in the string pool, the read
+       data are put in the upper half of the result, thus
+       the conversion to hex can be done without overwriting
+       unconverted bytes. */
+    data_ptr = poolptr + length;
+    read = fread(&strpool[data_ptr], sizeof(char), length, f);
+    fclose(f);
+
+    /* convert to hex */
+    data_end = data_ptr + read;
+    for (; data_ptr < data_end; data_ptr++) {
+        i = snprintf((char *) &strpool[poolptr], 3,
+                     "%.2X", (unsigned int) strpool[data_ptr]);
+        check_nprintf(i, 3);
+        poolptr += i;
+    }
+    xfree(file_name);
+}
+#endif /* e-pTeX or e-upTeX */
+#endif /* TeX */
 
 /* Metafont/MetaPost fraction routines. Replaced either by assembler or C.
    The assembler syntax doesn't work on Solaris/x86.  */
