@@ -48,12 +48,6 @@ typedef struct XeTeXLayoutEngine_rec* XeTeXLayoutEngine;
 #include FT_TRUETYPE_TABLES_H
 
 #include <hb.h>
-#include <hb-ot.h>
-#include <hb-icu.h>
-
-#include <graphite2/Font.h>
-#include <graphite2/Segment.h>
-#include <hb-graphite2.h>
 
 #include "XeTeX_ext.h"
 #include "XeTeXFontMgr.h"
@@ -62,7 +56,7 @@ typedef struct XeTeXLayoutEngine_rec* XeTeXLayoutEngine;
 extern "C" {
 #endif
 
-extern char	gPrefEngine;
+extern char gPrefEngine;
 
 int getCachedGlyphBBox(uint16_t fontID, uint16_t glyphID, GlyphBBox* bbox);
 void cacheGlyphBBox(uint16_t fontID, uint16_t glyphID, const GlyphBBox* bbox);
@@ -80,9 +74,8 @@ char getReqEngine();
 void setReqEngine(char reqEngine);
 const char* getFullName(PlatformFontRef fontRef);
 
-const char* getFontFilename(XeTeXLayoutEngine engine);
+const char* getFontFilename(XeTeXLayoutEngine engine, uint32_t* index);
 
-void getNames(PlatformFontRef fontRef, const char** psName, const char** famName, const char** styName);
 double getDesignSize(XeTeXFont font);
 
 void deleteFont(XeTeXFont font);
@@ -102,9 +95,9 @@ hb_tag_t getIndFeature(XeTeXFont font, hb_tag_t script, hb_tag_t language, unsig
 
 float getGlyphWidth(XeTeXFont font, uint32_t gid);
 
-XeTeXLayoutEngine createLayoutEngine(PlatformFontRef fontRef, XeTeXFont font, hb_tag_t script, hb_tag_t language,
-						hb_feature_t* features, int nFeatures, char **shapers, uint32_t rgbValue,
-						float extend, float slant, float embolden);
+XeTeXLayoutEngine createLayoutEngine(PlatformFontRef fontRef, XeTeXFont font, hb_tag_t script, char *language,
+                        hb_feature_t* features, int nFeatures, char **shapers, uint32_t rgbValue,
+                        float extend, float slant, float embolden);
 
 void deleteLayoutEngine(XeTeXLayoutEngine engine);
 
@@ -116,11 +109,11 @@ float getSlantFactor(XeTeXLayoutEngine engine);
 float getEmboldenFactor(XeTeXLayoutEngine engine);
 
 int layoutChars(XeTeXLayoutEngine engine, uint16_t* chars, int32_t offset, int32_t count, int32_t max,
-						bool rightToLeft);
+                        bool rightToLeft);
 
 void getGlyphs(XeTeXLayoutEngine engine, uint32_t* glyphs);
 void getGlyphAdvances(XeTeXLayoutEngine engine, float *advances);
-void getGlyphPositions(XeTeXLayoutEngine engine, float* positions);
+void getGlyphPositions(XeTeXLayoutEngine engine, FloatPoint* positions);
 
 float getPointSize(XeTeXLayoutEngine engine);
 
@@ -143,7 +136,7 @@ float getGlyphItalCorr(XeTeXLayoutEngine engine, uint32_t glyphID);
 
 uint32_t mapCharToGlyph(XeTeXLayoutEngine engine, uint32_t charCode);
 
-int	mapGlyphToIndex(XeTeXLayoutEngine engine, const char* glyphName);
+int mapGlyphToIndex(XeTeXLayoutEngine engine, const char* glyphName);
 
 const char* getGlyphName(XeTeXFont font, uint16_t gid, int* len);
 
